Package: ggsql
Title: Grammar of Graphics for SQL
Version: 0.3.3
Authors@R: c(
    person("Thomas Lin", "Pedersen", , "thomas.pedersen@posit.co",
           role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5147-4711")),
    person("George", "Stagg", , "george.stagg@posit.co", role = "aut",
           comment = c(ORCID = "0009-0006-3173-9846")),
    person("Teun", "van den Brand", , "teun.vandenbrand@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-9335-7468")),
    person("Posit, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Allows you to write queries that combine SQL (Structured Query
  Language) data retrieval with visualization specifications in a single,
  composable syntax. The 'ggsql' package binds directly with the 'ggsql' 'Rust'
  library and allows you to set up readers and writers and execute queries
  against it. The package also offers 'knitr' and 'shiny' integration allowing
  the user to use 'ggsql' in both frameworks.
License: MIT + file LICENSE
Encoding: UTF-8
SystemRequirements: Cargo (Rust's package manager), rustc (>= 1.86),
        ODBC driver manager (unixODBC on Linux, iODBC or unixODBC on
        macOS; built into Windows)
RoxygenNote: 7.3.3
Imports: cli, htmltools, htmlwidgets, jsonlite, knitr, nanoarrow, R6,
        rlang (>= 1.1.0), yaml
Suggests: gapminder, quarto, reticulate, rmarkdown, rsvg, shiny,
        testthat (>= 3.0.0), V8, withr
Config/testthat/edition: 3
Config/rextendr/version: 0.4.2.9000
Depends: R (>= 4.2)
VignetteBuilder: quarto
URL: https://r.ggsql.org, https://github.com/posit-dev/ggsql-r
Config/Needs/website: tidyverse/tidytemplate
Config/Needs/js: node
BugReports: https://github.com/posit-dev/ggsql-r/issues
NeedsCompilation: yes
Packaged: 2026-06-02 13:32:14 UTC; thomas
Author: Thomas Lin Pedersen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5147-4711>),
  George Stagg [aut] (ORCID: <https://orcid.org/0009-0006-3173-9846>),
  Teun van den Brand [aut] (ORCID:
    <https://orcid.org/0000-0002-9335-7468>),
  Posit, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Thomas Lin Pedersen <thomas.pedersen@posit.co>
Repository: CRAN
Date/Publication: 2026-06-03 08:20:23 UTC

# Generated by roxygen2: do not edit by hand

S3method("$",GgsqlReader)
S3method("$",GgsqlSpec)
S3method("$",GgsqlWriter)
S3method("$",ggsql_tables)
S3method("[[",GgsqlReader)
S3method("[[",GgsqlSpec)
S3method("[[",GgsqlWriter)
S3method("[[",ggsql_tables)
S3method(knit_print,Spec)
S3method(names,ggsql_tables)
S3method(print,ggsql_tables)
S3method(print,ggsql_validated)
S3method(str,Spec)
export(custom_reader)
export(duckdb_reader)
export(ggsqlOutput)
export(ggsql_execute)
export(ggsql_execute_sql)
export(ggsql_has_visual)
export(ggsql_is_valid)
export(ggsql_layer_count)
export(ggsql_layer_data)
export(ggsql_layer_sql)
export(ggsql_metadata)
export(ggsql_register)
export(ggsql_render)
export(ggsql_save)
export(ggsql_session_reader)
export(ggsql_sql)
export(ggsql_stat_data)
export(ggsql_stat_sql)
export(ggsql_table)
export(ggsql_table_names)
export(ggsql_unregister)
export(ggsql_validate)
export(ggsql_visual)
export(ggsql_warnings)
export(odbc_reader)
export(renderGgsql)
export(snowflake_reader)
export(vegalite_writer)
import(rlang)
importFrom(R6,R6Class)
importFrom(knitr,knit_print)
importFrom(utils,str)
useDynLib(ggsql, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ggsql <img src="man/figures/logo.png" align="right" height="138" alt="" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/posit-dev/ggsql-r/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/posit-dev/ggsql-r/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/posit-dev/ggsql-r/graph/badge.svg)](https://app.codecov.io/gh/posit-dev/ggsql-r)
<!-- badges: end -->

The ggsql R package provides rust bindings to the
[ggsql](https://ggsql.org) visualization tool so that you can hook up
readers and writers to it, execute queries, and visualize the result. It
also contain a knitr engine for supporting `ggsql` blocks, with
facilities for bidirectional data flow between R, Python, and ggsql
blocks. This means that you can prepare some data in one block using
dplyr or pandas, and then visualize it with ggsql in a different block
without having to do anything to pass the data around.

## Installation

ggsql can be installed from CRAN with:

``` r
# install.packages("pak")
pak::pak("ggsql")
```

To get the development version, change `"ggsql"` to `"posit-dev/ggsql"`.

## Example

While one of the core appeals of the ggsql R package is the knitr engine
it provides, you can also use it directly in R to execute visual
queries:

``` r
library(ggsql)

# Create an in-memory DuckDB reader
reader <- duckdb_reader()

# Register a dataset in it
ggsql_register(reader, mtcars, "amazing_data")

# Visualize it with a query
ggsql_execute(reader, "
VISUALIZE mpg AS x, disp AS y FROM amazing_data
DRAW point
")
```

<img src="man/figures/README-unnamed-chunk-2-1.svg" alt="An example plot created with ggsql through the R bindings. The plot shows a scatterplot of the mtcars dataset with mpg on the x axis and disp on the y axis" width="100%" />

We could achieve the same in a ggsql code block by referencing an R
dataset directly using the `r:` prefix

``` ggsql
VISUALIZE mpg AS x, disp AS y FROM r:mtcars
DRAW point
LABEL
  title => 'That data came from R 🤯'
```

<img src="man/figures/README-unnamed-chunk-3-1.svg" alt="An example plot created with ggsql referencing data from the R session. The plot shows a scatterplot of the mtcars dataset with mpg on the x axis and disp on the y axis" width="100%" />

The only thing you need to remember is to load ggsql into R in your
Rmarkdown/Quarto document so the knitr engine is registered.

## ----setup--------------------------------------------------------------------
library(ggsql)


## VISUALIZE species AS x FROM ggsql:penguins
## DRAW bar

## SELECT COUNT(*) AS count, species FROM ggsql:penguins
## GROUP BY species

## -----------------------------------------------------------------------------
gapminder_2002 <- gapminder::gapminder[gapminder::gapminder$year == 2002, ]


## VISUALIZE gdpPercap AS x, lifeExp AS y FROM r:gapminder_2002
## DRAW point
##   MAPPING continent AS stroke, pop AS size
##   SETTING fill => null
## LABEL
##   title => 'Rendering data from R'

## CREATE VIEW may_airquality AS
## SELECT * FROM ggsql:airquality
## WHERE Month = 5

## CREATE TABLE penguin_count AS
## SELECT COUNT(*) AS number, species FROM ggsql:penguins
## GROUP BY species

## -----------------------------------------------------------------------------
names(sql)

sql$penguin_count


## -----------------------------------------------------------------------------
#| include: false
# ggsql caches builtin datasets (penguins, airquality) as parquet files in
# the OS temp dir (Rust's `env::temp_dir()`, i.e. `$TMPDIR` or `/tmp`) and
# never cleans them up. Remove them so R CMD check doesn't flag them as
# detritus in the temp directory.
tmp_root <- Sys.getenv("TMPDIR", unset = "/tmp")
unlink(file.path(tmp_root, c("airquality.parquet", "penguins.parquet")))

# ---------------------------------------------------------------------------
# Server-side Vega-Lite rendering via V8
# ---------------------------------------------------------------------------

#' Save a ggsql spec to a file
#'
#' This function renders a specification and returns it either as a Vegalite
#' json string, an SVG or a PNG. For the latter two, the Vegalite JSON is
#' rendered to SVG using the V8 package and, potentially, converted to PNG using
#' the rsvg package.
#'
#' @param spec A `Spec` object returned by [ggsql_execute()].
#' @param file Output file path. Extension determines format: `.svg`, `.png`, or
#' `.json`.
#' @param width Width in pixels.
#' @param height Height in pixels.
#'
#' @return `file`, invisibly.
#'
#' @export
#'
#' @examples
#' reader <- duckdb_reader()
#' ggsql_register(reader, mtcars, "cars")
#' spec <- ggsql_execute(reader,
#'   "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
#' )
#' spec_file <- tempfile(fileext = ".json")
#' ggsql_save(spec, spec_file)
#'
ggsql_save <- function(spec, file, width = 600, height = 400) {
  ext <- tolower(tools::file_ext(file))
  switch(
    ext,
    svg = writeLines(ggsql_to_svg(spec, width, height), file),
    png = writeBin(ggsql_to_png(spec, width, height), file),
    json = writeLines(ggsql_render(vegalite_writer(), spec), file),
    cli::cli_abort("Unsupported format {.val {ext}}. Use svg, png, or json.")
  )
  invisible(file)
}

#' Render a ggsql spec to SVG
#'
#' Renders a visualization specification to an SVG string using Vega-Lite
#' via V8 (no browser required).
#'
#' @param spec A `Spec` object returned by [ggsql_execute()].
#' @param width Width in pixels.
#' @param height Height in pixels.
#' @return An SVG string (character).
#' @noRd
ggsql_to_svg <- function(spec, width = 600, height = 400) {
  rlang::check_installed("V8", reason = "to render Vega-Lite specs to SVG.")
  json <- ggsql_render(vegalite_writer(), spec)
  vegalite_to_svg(json, width = width, height = height)
}

#' Render a ggsql spec to PNG
#'
#' Renders a visualization specification to a PNG raw vector using Vega-Lite
#' via V8 and rsvg.
#'
#' @param spec A `Spec` object returned by [ggsql_execute()].
#' @param width Width in pixels.
#' @param height Height in pixels.
#' @return A raw vector containing PNG data.
#' @noRd
ggsql_to_png <- function(spec, width = 600, height = 400) {
  rlang::check_installed("rsvg", reason = "to render Vega-Lite specs to PNG.")
  svg <- ggsql_to_svg(spec, width = width, height = height)
  rsvg::rsvg_png(charToRaw(svg), width = width, height = height)
}

# ---------------------------------------------------------------------------
# V8 context management (cached, lazy-loaded)
# ---------------------------------------------------------------------------

get_vega_ctx <- function() {
  if (is.null(ggsql_env$v8_ctx)) {
    ggsql_env$v8_ctx <- create_vega_ctx()
  }
  ggsql_env$v8_ctx
}

create_vega_ctx <- function() {
  ctx <- V8::v8()

  # Polyfill APIs that Vega 6 expects but V8 doesn't provide
  ctx$eval(
    "
    if (typeof structuredClone === 'undefined') {
      globalThis.structuredClone = function(obj) {
        return JSON.parse(JSON.stringify(obj));
      };
    }
  "
  )

  # Load Vega libraries from CDN
  ctx$source("https://cdn.jsdelivr.net/npm/vega@6/build/vega.min.js")
  ctx$source("https://cdn.jsdelivr.net/npm/vega-lite@6/build/vega-lite.min.js")

  # Define helper function for spec → SVG conversion
  ctx$eval(
    "
    function renderToSvg(specJson, width, height) {
      var spec = JSON.parse(specJson);
      if (width) spec.width = width;
      if (height) spec.height = height;
      // Compile Vega-Lite to Vega
      var vegaSpec = vegaLite.compile(spec).spec;
      // Create a headless Vega view and render to SVG
      var view = new vega.View(vega.parse(vegaSpec), { renderer: 'none' });
      return view.toSVG();
    }
  "
  )

  ctx
}

vegalite_to_svg <- function(spec_json, width = NULL, height = NULL) {
  ctx <- get_vega_ctx()

  # Assign the spec as a variable to avoid quoting issues with large JSON
  ctx$assign("__spec_json__", spec_json)
  ctx$assign("__width__", width)
  ctx$assign("__height__", height)

  # toSVG() returns a Promise, use await
  ctx$eval("renderToSvg(__spec_json__, __width__, __height__)", await = TRUE)
}

# Note: Any variables prefixed with `.` are used for text
# replacement in the Makevars.in and Makevars.win.in

# check the packages MSRV first
source("tools/msrv.R")

# check DEBUG and NOT_CRAN environment variables
env_debug <- Sys.getenv("DEBUG")
env_not_cran <- Sys.getenv("NOT_CRAN")

# check if the vendored zip file exists
vendor_exists <- file.exists("src/rust/vendor.tar.xz")

is_not_cran <- env_not_cran != ""
is_debug <- env_debug != ""

if (is_debug) {
  # if we have DEBUG then we set not cran to true
  # CRAN is always release build
  is_not_cran <- TRUE
  message("Creating DEBUG build.")
}

if (!is_not_cran) {
  message("Building for CRAN.")
}

# we set cran flags only if NOT_CRAN is empty and if
# the vendored crates are present.
.cran_flags <- ifelse(
  !is_not_cran && vendor_exists,
  "-j 2 --offline",
  ""
)

# when DEBUG env var is present we use `--debug` build
.profile <- ifelse(is_debug, "", "--release")
.clean_targets <- ifelse(is_debug, "", "$(TARGET_DIR)")

# We specify this target when building for webR
webr_target <- "wasm32-unknown-emscripten"

# here we check if the platform we are building for is webr
is_wasm <- identical(R.version$platform, webr_target)

# print to terminal to inform we are building for webr
if (is_wasm) {
  message("Building for WebR")
}

# we check if we are making a debug build or not
# if so, the LIBDIR environment variable becomes:
# LIBDIR = $(TARGET_DIR)/{wasm32-unknown-emscripten}/debug
# this will be used to fill out the LIBDIR env var for Makevars.in
target_libpath <- if (is_wasm) "wasm32-unknown-emscripten" else NULL
cfg <- if (is_debug) "debug" else "release"

# used to replace @LIBDIR@
.libdir <- paste(c(target_libpath, cfg), collapse = "/")

# use this to replace @TARGET@
# we specify the target _only_ on webR
# there may be use cases later where this can be adapted or expanded
.target <- ifelse(is_wasm, paste0("--target=", webr_target), "")

# add panic exports only for WASM builds
.panic_exports <- ifelse(
  is_wasm,
  "CARGO_PROFILE_DEV_PANIC=\"abort\" CARGO_PROFILE_RELEASE_PANIC=\"abort\" ",
  ""
)

# On macOS, forward a deployment target into the cargo build so cc-rs
# compiles C/C++ deps (notably libduckdb-sys) with -mmacosx-version-min
# no newer than R's link step. Without this, cc defaults to the host SDK's
# max version and R CMD check flags a "newer macOS version than being
# linked" warning. Resolution order: explicit env var, then R's CFLAGS,
# then a conservative floor of 11.0 (the Apple Silicon minimum) which is
# older than any modern macOS so the linker won't complain.
.macos_target <- ""
if (Sys.info()[["sysname"]] == "Darwin") {
  dt <- Sys.getenv("MACOSX_DEPLOYMENT_TARGET", unset = "")
  if (!nzchar(dt)) {
    cflags <- tryCatch(
      suppressWarnings(system2(
        file.path(R.home("bin"), "R"),
        c("CMD", "config", "CFLAGS"),
        stdout = TRUE,
        stderr = FALSE
      )),
      error = function(e) character()
    )
    m <- regmatches(
      cflags,
      regexpr("-mmacosx-version-min=([0-9.]+)", cflags)
    )
    if (length(m) > 0) {
      dt <- sub("-mmacosx-version-min=", "", m[[1]])
    }
  }
  if (!nzchar(dt)) {
    dt <- "11.0"
  }
  .macos_target <- paste0("MACOSX_DEPLOYMENT_TARGET=", dt, " ")
}

# Resolve ODBC driver manager flags (unixODBC or iODBC). The ggsql Rust
# crate dynamically loads the ODBC driver manager via `libloading` (dlopen)
# at runtime, so `-lodbc` is not required at link time -- cargo's
# `native-static-libs` print confirms this. We therefore default to empty
# and only add flags when pkg-config can actually locate an ODBC .pc file.
#
# When pkg-config finds odbc, we keep only `-L/-l` tokens from its output:
# some iODBC .pc files (notably the one shipped on the CRAN sonoma-arm64
# builder) embed a stray `libltdl/libltdlc.la` libtool archive path in
# their Libs line that the linker can't resolve. Restricting to `-L`/`-l`
# strips those out.
#
# Hard-coding a `-lodbc` fallback would break platforms where libodbc is
# not on the linker's default search path (e.g. CRAN's R-devel M1 builder),
# and is unnecessary because the Rust side resolves ODBC symbols at
# runtime, not link time.
.odbc_libs <- tryCatch(
  {
    out <- suppressWarnings(system2(
      "pkg-config",
      c("--libs", "odbc"),
      stdout = TRUE,
      stderr = FALSE
    ))
    if (!is.null(attr(out, "status")) || length(out) == 0) {
      ""
    } else {
      tokens <- unlist(strsplit(paste(out, collapse = " "), "\\s+"))
      tokens <- tokens[nzchar(tokens)]
      tokens <- tokens[grepl("^-[Ll]", tokens)]
      paste(tokens, collapse = " ")
    }
  },
  error = function(e) ""
)

# read in the Makevars.in file checking
is_windows <- .Platform[["OS.type"]] == "windows"

# if windows we replace in the Makevars.win.in
mv_fp <- ifelse(
  is_windows,
  "src/Makevars.win.in",
  "src/Makevars.in"
)

# set the output file
mv_ofp <- ifelse(
  is_windows,
  "src/Makevars.win",
  "src/Makevars"
)

# delete the existing Makevars{.win/.wasm}
if (file.exists(mv_ofp)) {
  message("Cleaning previous `", mv_ofp, "`.")
  invisible(file.remove(mv_ofp))
}

# read as a single string
mv_txt <- readLines(mv_fp)

# replace placeholder values
new_txt <- gsub("@CRAN_FLAGS@", .cran_flags, mv_txt) |>
  gsub("@PROFILE@", .profile, x = _) |>
  gsub("@CLEAN_TARGET@", .clean_targets, x = _) |>
  gsub("@LIBDIR@", .libdir, x = _) |>
  gsub("@TARGET@", .target, x = _) |>
  gsub("@PANIC_EXPORTS@", .panic_exports, x = _) |>
  gsub("@MACOSX_TARGET@", .macos_target, x = _) |>
  gsub("@ODBC_LIBS@", .odbc_libs, x = _)

message("Writing `", mv_ofp, "`.")
con <- file(mv_ofp, open = "wb")
writeLines(new_txt, con, sep = "\n")
close(con)

message("`tools/config.R` has finished.")

.onLoad <- function(lib, pkg) {
  run_on_load()
}

 .onUnload <- function(libpath) {
   unlink(file.path(tempdir(), c("penguins.parquet", "airquality.parquet")))
 }

release_bullets <- function() {
  c(
    "Bump rust version to match package version",
    "Re-vendor rust dependencies",
    "Run `rextendr::document()` to refresh extendr wrappers and Rd files",
    "Run `npm run check` to rebuild `inst/htmlwidgets/ggsql_vega.js` from `srcts/`",
    "Create the GitHub Release for this version *before* submitting to CRAN (`gh release create v<version> --notes-file NEWS.md --target main`) so the `release-vendor.yaml` workflow can attach `vendor.tar.xz` + `vendor.tar.xz.sha256` to it",
    "Confirm the `release-vendor.yaml` workflow run finished and both assets are listed on the release page",
    "Smoke-test the externalized vendor path: in a clean checkout without `src/rust/vendor.tar.xz` or `src/rust/vendor/`, run `R CMD build .` and `R CMD INSTALL` the resulting tarball - install should download the archive from the release"
  )
}

#' @noRd
ggsql_widget <- function(
  spec_json,
  width = NULL,
  height = NULL
) {
  widget <- htmlwidgets::createWidget(
    name = "ggsql_vega",
    x = list(
      spec = jsonlite::parse_json(spec_json)
    ),
    width = width,
    height = height,
    sizingPolicy = htmlwidgets::sizingPolicy(
      viewer.fill = TRUE,
      browser.fill = TRUE,
      knitr.figure = TRUE,
      knitr.defaultWidth = "100%",
      knitr.defaultHeight = "400px"
    ),
    package = "ggsql"
  )
  widget$dependencies <- c(
    widget$dependencies,
    vega_dependencies(),
    widget_dependencies()
  )
  widget
}

vega_dependencies <- function() {
  list(
    htmltools::htmlDependency(
      name = "vega",
      version = vega_version,
      src = "htmlwidgets/lib/vega",
      package = "ggsql",
      script = "vega.min.js"
    ),
    htmltools::htmlDependency(
      name = "vega-lite",
      version = vega_lite_version,
      src = "htmlwidgets/lib/vega-lite",
      package = "ggsql",
      script = "vega-lite.min.js"
    ),
    htmltools::htmlDependency(
      name = "vega-embed",
      version = vega_embed_version,
      src = "htmlwidgets/lib/vega-embed",
      package = "ggsql",
      script = "vega-embed.min.js"
    )
  )
}

widget_dependencies <- function() {
  list(
    htmltools::htmlDependency(
      name = "ggsql-vega-styles",
      version = utils::packageVersion("ggsql"),
      src = "htmlwidgets",
      package = "ggsql",
      stylesheet = "ggsql_vega.css",
      all_files = FALSE
    )
  )
}

#' @noRd
widget_html.ggsql_vega <- function(
  name,
  package,
  id,
  style,
  class,
  inline = FALSE,
  ...
) {
  htmltools::tag(
    "ggsql-vega",
    list(
      id = id,
      style = paste0("display:block;", style),
      class = class
    )
  )
}

#' Set the ggsql reader for the current Shiny session
#'
#' Registers a [duckdb_reader()] for use by all [renderGgsql()] outputs in the
#' current Shiny session. Must be called from within a Shiny server function
#' (i.e., while a session is active). The reader is automatically cleaned up
#' when the session ends.
#'
#' @param reader A `Reader` object created by [duckdb_reader()].
#' @param session The Shiny session object. Defaults to the current session.
#'
#' @return The `reader`, invisibly.
#'
#' @export
#'
#' @examplesIf interactive() && requireNamespace("shiny", quietly = TRUE)
#' library(shiny)
#'
#' ui <- fluidPage(
#'   ggsqlOutput("chart")
#' )
#'
#' server <- function(input, output, session) {
#'   ggsql_session_reader(duckdb_reader())
#'
#'   output$chart <- renderGgsql({
#'     "SELECT * FROM r:mtcars VISUALISE mpg AS x, disp AS y DRAW point"
#'   })
#' }
#'
#' shinyApp(ui, server)
ggsql_session_reader <- function(
  reader,
  session = shiny::getDefaultReactiveDomain()
) {
  rlang::check_installed("shiny", reason = "for ggsql Shiny bindings.")
  if (is.null(session)) {
    cli::cli_abort(
      "{.fn ggsql_session_reader} must be called from within a Shiny server function."
    )
  }
  session$userData$.ggsql_reader <- reader
  # Drop the reference so GC can invoke the Rust finalizer on the DuckDB
  # connection. Deterministic $close() doesn't exist yet on Reader.
  session$onSessionEnded(function() {
    session$userData$.ggsql_reader <- NULL
  })
  invisible(reader)
}

get_session_reader <- function(session) {
  session$userData$.ggsql_reader
}

#' Shiny bindings for ggsql
#'
#' Render ggsql visualizations in a Shiny application. `renderGgsql()` accepts
#' either a ggsql query string or a `Spec` object (returned by
#' [ggsql_execute()]). When given a string, it validates and executes the query
#' against `reader`.
#'
#' @param outputId Output variable to read from.
#' @param width,height CSS dimensions for the output container.
#'
#' @return `ggsqlOutput()` returns a Shiny UI element. `renderGgsql()` returns
#' a Shiny render function.
#'
#' @export
#'
#' @examplesIf interactive() && requireNamespace("shiny", quietly = TRUE)
#' library(shiny)
#'
#' ui <- fluidPage(
#'   ggsqlOutput("chart")
#' )
#'
#' server <- function(input, output, session) {
#'   ggsql_session_reader(duckdb_reader())
#'
#'   output$chart <- renderGgsql({
#'     "SELECT * FROM r:mtcars VISUALISE mpg AS x, disp AS y DRAW point"
#'   })
#' }
#'
#' shinyApp(ui, server)
ggsqlOutput <- function(outputId, width = "100%", height = "400px") {
  htmlwidgets::shinyWidgetOutput(
    outputId,
    name = "ggsql_vega",
    width = width,
    height = height,
    package = "ggsql"
  )
}

#' @param expr An expression that returns a ggsql query string or a `Spec`
#'   object. Strings may contain `r:varname` references that resolve variables
#'   from the expression's local scope (see Examples).
#' @param reader A `Reader` object created by [duckdb_reader()]. When `NULL`
#'   (the default), the session reader set by [ggsql_session_reader()] is used.
#' @param env The environment in which to evaluate `expr`.
#' @param quoted Logical. Is `expr` a quoted expression?
#'
#' @rdname ggsqlOutput
#' @export
renderGgsql <- function(
  expr,
  reader = NULL,
  env = parent.frame(),
  quoted = FALSE
) {
  if (!quoted) {
    expr <- substitute(expr)
  }
  force(env)

  render_expr <- quote({
    eval_env <- new.env(parent = env)
    value <- eval(expr, envir = eval_env)

    if (inherits(value, "htmlwidget")) {
      return(value)
    }

    if (inherits(value, "Spec")) {
      json <- ggsql_render(vegalite_writer(), value)
      return(ggsql_widget(json))
    }

    if (!is.character(value) || length(value) != 1L) {
      cli::cli_abort(
        "Expected a ggsql query string or a {.cls Spec} object, not {.cls {class(value)}}."
      )
    }

    session <- shiny::getDefaultReactiveDomain()
    r <- reader %||% get_session_reader(session)
    if (is.null(r)) {
      cli::cli_abort(
        c(
          "No ggsql reader available.",
          i = "Call {.code ggsql_session_reader(duckdb_reader())} in your server function before using {.fn renderGgsql}.",
          i = "Or pass a {.arg reader} argument directly to {.fn renderGgsql}."
        )
      )
    }
    query <- resolve_data_refs(value, r, envir = eval_env)
    validated <- ggsql_validate(query)
    if (!validated$has_visual) {
      cli::cli_abort(
        "{.fn renderGgsql} only accepts ggsql queries with a {.code VISUALISE} clause."
      )
    }
    spec <- ggsql_execute(r, query)
    json <- ggsql_render(vegalite_writer(), spec)
    ggsql_widget(json)
  })

  htmlwidgets::shinyRenderWidget(
    render_expr,
    ggsqlOutput,
    env = environment(),
    quoted = TRUE
  )
}

test_that("renderGgsql does not accept output sizing args", {
  skip_if_not_installed("shiny")

  expect_false("..." %in% names(formals(renderGgsql)))
  expect_identical(
    names(formals(renderGgsql)),
    c("expr", "reader", "env", "quoted")
  )
})

test_that("ggsqlOutput owns output sizing", {
  skip_if_not_installed("shiny")

  output <- ggsqlOutput("chart", width = "123px", height = "456px")
  output_html <- paste(capture.output(print(output)), collapse = "\n")

  expect_match(
    output_html,
    'style="display:block;width:123px;height:456px;"',
    fixed = TRUE
  )
})

test_that("renderGgsql rejects plain SQL strings before execution", {
  skip_if_not_installed("shiny")

  reader <- duckdb_reader()
  render_fn <- renderGgsql(
    {
      "SELECT 1 AS x"
    },
    reader = reader
  )

  expect_error(
    environment(render_fn)$origUserFunc(),
    "`renderGgsql()` only accepts ggsql queries with a `VISUALISE` clause.",
    fixed = TRUE
  )
})

#' Create a Vega-Lite writer
#'
#' This function creates a vegalite writer which is currently the only writer
#' type for ggsql
#'
#' @return A `Writer` object.
#'
#' @export
#'
#' @examples
#' vegalite_writer()
#'
vegalite_writer <- function() {
  Writer$new()
}

#' @noRd
Writer <- R6::R6Class(
  "Writer",
  cloneable = FALSE,
  public = list(
    .ptr = NULL,

    initialize = function() {
      self$.ptr <- GgsqlWriter$new()
    },

    print = function(...) {
      cli::cli_text("<ggsql_writer> [vegalite]")
      invisible(self)
    }
  )
)

#' Render a spec with a writer
#'
#' This function takes a `Spec` object as returned by [ggsql_execute()] and
#' renders it with the provided writer.
#'
#' @param writer A `Writer` object created by e.g. [vegalite_writer()].
#' @param spec A `Spec` object returned by [ggsql_execute()].
#'
#' @return Writer dependent:
#'
#' * `vegalite_writer`: A string holding the vegalite JSON representation of the
#' visualization
#'
#' @export
#'
#' @examples
#' reader <- duckdb_reader()
#' ggsql_register(reader, mtcars, "cars")
#' spec <- ggsql_execute(reader,
#'   "SELECT * FROM cars VISUALISE mpg AS x DRAW histogram"
#' )
#'
#' ggsql_render(vegalite_writer(), spec)
#'
ggsql_render <- function(writer, spec) {
  check_r6(writer, "Writer")
  check_r6(spec, "Spec")
  writer$.ptr$render(spec$.ptr)
}

# Package-level environment for persistent reader across knitr chunks
ggsql_env <- new.env(parent = emptyenv())

get_engine_reader <- function(connection = NULL) {
  if (is.null(connection)) {
    # Default in-memory DuckDB reader
    if (is.null(ggsql_env$reader)) {
      ggsql_env$reader <- duckdb_reader()
      # Inject the sql proxy into the knit environment for cross-chunk access
      proxy <- new_ggsql_tables()
      knit_env <- knitr::knit_global()
      knit_env[["sql"]] <- proxy
      # Also inject into Python if reticulate is available, so Python chunks
      # can use sql.tablename directly (without the r. prefix)
      if (
        requireNamespace("reticulate", quietly = TRUE) &&
          reticulate::py_available(initialize = FALSE)
      ) {
        reticulate::py_run_string("pass") # ensure Python is initialized
        py <- reticulate::py
        py[["sql"]] <- proxy
      }
    }
    return(ggsql_env$reader)
  }

  # `custom://` always resolves fresh — the user owns the Reader and may
  # rebind the variable across chunks; caching would shadow that.
  if (grepl("^custom://", connection, ignore.case = TRUE)) {
    return(parse_connection(connection))
  }

  # Other connections: cache by connection string
  if (is.null(ggsql_env$readers)) {
    ggsql_env$readers <- list()
  }
  if (is.null(ggsql_env$readers[[connection]])) {
    ggsql_env$readers[[connection]] <- parse_connection(connection)
  }
  ggsql_env$readers[[connection]]
}

register_syntax_highlighting <- function() {
  syntax_file <- system.file("ggsql.xml", package = "ggsql")
  if (!nzchar(syntax_file)) {
    return()
  }

  # For rmarkdown: add --syntax-definition to Pandoc args
  current <- knitr::opts_knit$get("rmarkdown.pandoc.args")
  if (!syntax_file %in% current) {
    knitr::opts_knit$set(
      rmarkdown.pandoc.args = c(current, "--syntax-definition", syntax_file)
    )
  }
}

# ---------------------------------------------------------------------------
# Connection string parsing
# ---------------------------------------------------------------------------

parse_connection <- function(connection) {
  check_string(connection)

  if (!grepl("://", connection, fixed = TRUE)) {
    cli::cli_abort(
      "Invalid connection string {.val {connection}}. Expected format: {.code scheme://...} (e.g., {.code duckdb://memory})."
    )
  }

  scheme <- sub("://.*", "", connection)

  switch(
    tolower(scheme),
    duckdb = Reader$new(connection),
    odbc = Reader$new(connection),
    snowflake = Reader$new(connection),
    custom = resolve_custom_reader(connection),
    cli::cli_abort(
      "Unsupported connection scheme {.val {scheme}}. Supported schemes: {.val duckdb}, {.val odbc}, {.val snowflake}, {.val custom}."
    )
  )
}

resolve_custom_reader <- function(connection) {
  name <- sub("^custom://", "", connection, ignore.case = TRUE)
  if (!nzchar(name)) {
    cli::cli_abort(c(
      "Invalid custom connection {.val {connection}}.",
      i = "Expected {.code custom://name}, where {.var name} is a {.cls Reader} in the knit environment."
    ))
  }
  env <- knitr::knit_global()
  if (!exists(name, envir = env, inherits = FALSE)) {
    cli::cli_abort(
      "Custom reader {.val {name}} not found in the knit environment."
    )
  }
  obj <- get(name, envir = env, inherits = FALSE)
  if (!inherits(obj, "Reader")) {
    cli::cli_abort(
      "{.var {name}} is not a {.cls Reader} (got {.cls {class(obj)[1]}})."
    )
  }
  obj
}

# ---------------------------------------------------------------------------
# sql proxy: live access to tables in the ggsql DuckDB reader
# ---------------------------------------------------------------------------

new_ggsql_tables <- function() {
  structure(list(), class = "ggsql_tables")
}

#' @export
`$.ggsql_tables` <- function(x, name) {
  safe_name <- gsub('"', '""', name, fixed = TRUE)
  ggsql_execute_sql(
    get_engine_reader(knitr::opts_current$get("connection")),
    paste0('SELECT * FROM "', safe_name, '"')
  )
}

#' @export
`[[.ggsql_tables` <- function(x, name, ...) {
  safe_name <- gsub('"', '""', name, fixed = TRUE)
  ggsql_execute_sql(
    get_engine_reader(knitr::opts_current$get("connection")),
    paste0('SELECT * FROM "', safe_name, '"')
  )
}

#' @export
print.ggsql_tables <- function(x, ...) {
  reader <- get_engine_reader(knitr::opts_current$get("connection"))
  tables <- try_fetch(
    ggsql_execute_sql(reader, "SHOW TABLES"),
    error = function(cnd) data.frame(name = character())
  )
  cli::cli_text("<ggsql tables>")
  if (nrow(tables) > 0) {
    cli::cli_bullets(stats::setNames(tables[[1]], rep("*", nrow(tables))))
  } else {
    cli::cli_text("(no tables)")
  }
  invisible(x)
}

#' @export
names.ggsql_tables <- function(x) {
  reader <- get_engine_reader(knitr::opts_current$get("connection"))
  tables <- try_fetch(
    ggsql_execute_sql(reader, "SHOW TABLES"),
    error = function(cnd) data.frame(name = character())
  )
  tables <- if (nrow(tables) > 0) tables[[1]] else character()
  tables[!grepl("^__(ggsql|r|py)_", tables, perl = TRUE)]
}

# ---------------------------------------------------------------------------
# Data reference resolution (r: and py: prefixes)
# ---------------------------------------------------------------------------

resolve_data_refs <- function(query, reader, envir) {
  refs <- gregexpr(
    "(?:r|py):[a-zA-Z_][a-zA-Z0-9_.]*",
    query,
    ignore.case = TRUE,
    perl = TRUE
  )
  matches <- regmatches(query, refs)[[1]]

  if (length(matches) == 0) {
    return(query)
  }

  for (ref in unique(matches)) {
    parts <- strsplit(ref, ":", fixed = TRUE)[[1]]
    prefix <- parts[1]
    name <- parts[2]

    df <- switch(
      tolower(prefix),
      r = try_fetch(
        get(name, envir = envir),
        error = function(cnd) {
          cli::cli_abort(
            "Data reference {.code {ref}}: object {.val {name}} not found in R environment."
          )
        }
      ),
      py = {
        rlang::check_installed(
          "reticulate",
          reason = "to use py: data references."
        )
        obj <- reticulate::py[[name]]
        if (is.null(obj)) {
          cli::cli_abort(
            "Data reference {.code {ref}}: object {.val {name}} not found in Python environment."
          )
        }
        obj
      }
    )

    if (!is.data.frame(df)) {
      cli::cli_abort(
        "{.code {ref}} refers to a {.cls {class(df)}} object, not a data frame."
      )
    }

    internal_name <- paste0(
      "__",
      prefix,
      "_",
      gsub(".", "_ggsqldot_", name, fixed = TRUE),
      "__"
    )
    ggsql_register(reader, df, internal_name, replace = TRUE)
    query <- gsub(ref, internal_name, query, fixed = TRUE)
  }

  query
}

# ---------------------------------------------------------------------------
# Inline chunk options (--| and #| prefix support)
# ---------------------------------------------------------------------------

parse_chunk_options <- function(options) {
  code <- options$code
  if (length(code) == 0) {
    return(options)
  }

  # Extract leading lines with --| or #| prefix
  option_pattern <- "^\\s*(?:--|#)\\|\\s?"
  is_option <- grepl(option_pattern, code, perl = TRUE)

  # Only consider contiguous leading option lines
  n_options <- 0L
  for (i in seq_along(is_option)) {
    if (!is_option[i]) {
      break
    }
    n_options <- i
  }

  if (n_options == 0L) {
    return(options)
  }

  # Extract option text (strip the prefix)
  option_lines <- sub(option_pattern, "", code[seq_len(n_options)], perl = TRUE)
  options$code <- code[-seq_len(n_options)]

  # Parse as YAML
  yaml_text <- paste(option_lines, collapse = "\n")
  parsed <- tryCatch(
    yaml::yaml.load(yaml_text),
    error = function(cnd) NULL
  )

  if (is.null(parsed) || !is.list(parsed)) {
    return(options)
  }

  # Merge into options (inline options override chunk header options)
  for (nm in names(parsed)) {
    # Convert Quarto-style kebab-case to knitr-style dot-case
    knitr_nm <- gsub("-", ".", nm, fixed = TRUE)
    options[[knitr_nm]] <- parsed[[nm]]
  }

  options
}

# ---------------------------------------------------------------------------
# knitr engine
# ---------------------------------------------------------------------------

ggsql_engine <- function(options) {
  # Parse inline chunk options (--| or #| prefixed lines)
  options <- parse_chunk_options(options)

  # Use SQL syntax highlighting for the source code block.
  # ggsql-specific highlighting requires adding ggsql.xml to the Quarto/Pandoc
  # config (see inst/ggsql.xml). SQL covers the base keywords well.
  options$class.source <- options$class.source %||% "sql"

  # Always register syntax highlighting, even for custom connections
  register_syntax_highlighting()

  if (!options$eval) {
    return(knitr::engine_output(options, options$code, ""))
  }

  query <- paste(options$code, collapse = "\n")

  result <- try_fetch(
    {
      reader <- get_engine_reader(options$connection)
      ggsql_engine_eval(query, reader, options)
    },
    error = function(cnd) {
      knitr::engine_output(options, options$code, conditionMessage(cnd))
    }
  )

  result
}

ggsql_engine_eval <- function(query, reader, options) {
  query <- resolve_data_refs(query, reader, envir = knitr::knit_global())
  validated <- ggsql_validate(query)

  if (!validated$has_visual) {
    # Plain SQL: execute and render as table
    df <- ggsql_execute_sql(reader, query)

    # If output.var is set, assign to knit environment instead of rendering
    if (!is.null(options$output.var)) {
      knit_env <- knitr::knit_global()
      knit_env[[options$output.var]] <- df
      return(knitr::engine_output(options, options$code, ""))
    }

    # DDL/DML statements (CREATE, INSERT, ...) return NULL — nothing to render.
    if (is.null(df) || nrow(df) == 0) {
      return(knitr::engine_output(options, options$code, ""))
    }
    out <- knitr::kable(df)
    options$results <- "asis"
    return(knitr::engine_output(options, options$code, out))
  }

  # Visualization query: execute and render
  spec <- ggsql_execute(reader, query)
  writer_type <- options$writer %||%
    if (knitr::is_latex_output()) "vegalite_png" else "vegalite"

  # If output.var is set, always capture the Vega-Lite JSON
  if (!is.null(options$output.var)) {
    writer <- vegalite_writer()
    json <- ggsql_render(writer, spec)
    knit_env <- knitr::knit_global()
    knit_env[[options$output.var]] <- json
    return(knitr::engine_output(options, options$code, ""))
  }

  options$results <- "asis"

  switch(
    writer_type,
    vegalite = {
      if (!is.null(options$fig.cap) && nzchar(options$fig.cap)) {
        cli::cli_warn(c(
          "{.code fig.cap} is not supported for interactive HTML output.",
          i = "Use {.code writer = \"vegalite_svg\"} or {.code writer = \"vegalite_png\"} for captioned figures."
        ))
      }
      writer <- vegalite_writer()
      json <- ggsql_render(writer, spec)
      widget <- ggsql_widget(json)
      out <- knitr::knit_print(widget, options = options)
      knitr::knit_meta_add(attr(out, "knit_meta"))
      knitr::engine_output(options, options$code, out = out)
    },
    vegalite_svg = render_static_figure(spec, "svg", options),
    vegalite_png = render_static_figure(spec, "png", options),
    cli::cli_abort(
      c(
        "Unsupported writer {.val {writer_type}}.",
        i = "Supported writers: {.val vegalite}, {.val vegalite_svg}, {.val vegalite_png}."
      )
    )
  )
}

write_static_figure <- function(spec, format, options) {
  options$label <- options$label %||% "ggsql-chunk"
  ext <- options$fig.ext %||% paste0(".", format)
  fig <- knitr::fig_path(ext, options)
  dir.create(dirname(fig), recursive = TRUE, showWarnings = FALSE)

  if (is.null(options$fig.dim)) {
    width <- options$fig.width * options$dpi
    height <- if (is.null(options$fig.asp)) {
      options$fig.height * options$dpi
    } else {
      width * options$fig.asp
    }
  } else {
    width <- options$fig.dim[1] * options$dpi
    height <- options$fig.dim[2] * options$dpi
  }

  switch(
    format,
    svg = writeLines(ggsql_to_svg(spec, width, height), fig),
    png = writeBin(ggsql_to_png(spec, width, height), fig)
  )

  knitr::include_graphics(fig)
}

render_static_figure <- function(spec, format, options) {
  out <- write_static_figure(spec, format, options)
  knitr::engine_output(options, options$code, knitr::sew(out, options))
}

on_load(
  knitr::knit_engines$set(ggsql = ggsql_engine)
)

# Acquire the vendored Rust crate archive before cargo runs.
#
# The vendored Rust crate archive is not shipped in the source tarball
# (CRAN ask); instead it is hosted per-version as a GitHub Release asset
# and downloaded here. After this script runs, `src/rust/vendor.tar.xz`
# exists and the existing Makevars logic extracts it for an `--offline`
# cargo build.
#
# Decision tree:
#   1. `GGSQL_VENDOR_TARBALL` env var set -> use that local path.
#   2. `src/rust/vendor/` already extracted -> nothing to do.
#   3. `src/rust/vendor.tar.xz` already present (local dev, or left over
#      from a previous configure run in this source tree) -> nothing to do.
#   4. `NOT_CRAN` set -> skip fetch; cargo will fetch crates online.
#   5. Otherwise download from the version-keyed GitHub Release into the
#      session tempdir, verify against the sidecar `vendor.tar.xz.sha256`,
#      and move into `src/rust/`.
#
# Nothing is written outside the source tree or the session tempdir, in
# line with CRAN's policy on install-time filesystem use.
#
# Env vars:
#   GGSQL_VENDOR_TARBALL  Path to a local archive (bypasses download).
#                         Useful for offline / proxied installs and CI.
#   GGSQL_VENDOR_URL      Override the primary archive URL (e.g. mirror or
#                         pre-release tag).
#   NOT_CRAN              If non-empty, skip vendor fetch entirely.

local({
  pkg_root <- getwd()
  vendor_xz <- file.path(pkg_root, "src", "rust", "vendor.tar.xz")
  vendor_dir <- file.path(pkg_root, "src", "rust", "vendor")

  desc <- read.dcf(file.path(pkg_root, "DESCRIPTION"))
  pkg_version <- unname(desc[, "Version"])

  sha256 <- function(path) {
    unname(tools::sha256sum(path))
  }

  copy <- function(src, dest) {
    if (!file.copy(src, dest, overwrite = TRUE)) {
      stop(sprintf("Failed to copy '%s' to '%s'.", src, dest), call. = FALSE)
    }
  }

  # 1. Explicit local-path override.
  override <- Sys.getenv("GGSQL_VENDOR_TARBALL")
  if (nzchar(override)) {
    if (!file.exists(override)) {
      stop(
        "GGSQL_VENDOR_TARBALL points at '", override, "' which does not exist.",
        call. = FALSE
      )
    }
    message("Using vendor archive from GGSQL_VENDOR_TARBALL: ", override)
    copy(override, vendor_xz)
    return(invisible())
  }

  # 2. Already-extracted vendor directory.
  if (dir.exists(vendor_dir)) {
    message("Vendor directory already extracted at src/rust/vendor; skipping fetch.")
    return(invisible())
  }

  # 3. Archive already present (local dev with rextendr::vendor_pkgs(), or
  # leftover from a previous configure run in the same source tree).
  if (file.exists(vendor_xz)) {
    message("Vendor archive already at src/rust/vendor.tar.xz; skipping fetch.")
    return(invisible())
  }

  # 4. NOT_CRAN: skip vendor fetch and let cargo go online.
  if (nzchar(Sys.getenv("NOT_CRAN"))) {
    message(
      "NOT_CRAN is set; skipping vendor archive fetch. ",
      "cargo will fetch crates from the network."
    )
    return(invisible())
  }

  # 5. Download from GitHub Release into session tempdir.
  default_base <- sprintf(
    "https://github.com/posit-dev/ggsql-r/releases/download/v%s",
    pkg_version
  )
  base_url <- Sys.getenv("GGSQL_VENDOR_URL", unset = default_base)
  archive_url <- paste0(base_url, "/vendor.tar.xz")
  sha_url <- paste0(base_url, "/vendor.tar.xz.sha256")

  download <- function(url, dest) {
    status <- tryCatch(
      utils::download.file(url, dest, mode = "wb", quiet = FALSE),
      error = function(e) {
        unlink(dest)
        stop(
          "Failed to download vendor archive from\n  ", url,
          "\nReason: ", conditionMessage(e),
          "\n\nTo bypass network fetch, set one of:\n",
          "  GGSQL_VENDOR_TARBALL=/path/to/vendor.tar.xz   (use a local archive)\n",
          "  GGSQL_VENDOR_URL=https://example/v", pkg_version, " (override URL)\n",
          "  NOT_CRAN=true                                  (skip fetch, build online)",
          call. = FALSE
        )
      }
    )
    if (!identical(status, 0L) || !file.exists(dest) || file.size(dest) == 0L) {
      unlink(dest)
      stop(
        "Download from\n  ", url,
        "\nreturned no usable file.",
        call. = FALSE
      )
    }
  }

  sha_tmp <- tempfile(fileext = ".sha256")
  archive_tmp <- tempfile(fileext = ".tar.xz")
  on.exit(unlink(c(sha_tmp, archive_tmp)), add = TRUE)

  download(sha_url, sha_tmp)
  expected_sha <- sub("\\s.*$", "", readLines(sha_tmp, warn = FALSE)[[1]])
  if (!grepl("^[0-9a-fA-F]{64}$", expected_sha)) {
    stop(
      "Sidecar checksum at\n  ", sha_url,
      "\nis not a valid 64-character hex sha256.",
      call. = FALSE
    )
  }

  message("Downloading vendor archive from ", archive_url)
  download(archive_url, archive_tmp)

  got_sha <- sha256(archive_tmp)
  if (!identical(got_sha, expected_sha)) {
    stop(
      "SHA256 mismatch for vendor archive downloaded from\n  ", archive_url,
      "\nExpected: ", expected_sha,
      "\nGot:      ", got_sha,
      call. = FALSE
    )
  }

  copy(archive_tmp, vendor_xz)
  message("Vendor archive placed at ", vendor_xz, " (", file.size(vendor_xz), " bytes)")
})

# Generated by extendr: Do not edit by hand

# nolint start

#
# This file was created with the following call:
#   .Call("wrap__make_ggsql_wrappers", use_symbols = TRUE, package_name = "ggsql")

#' @usage NULL
#' @useDynLib ggsql, .registration = TRUE
NULL

ggsql_validate_impl <- function(query) .Call(wrap__ggsql_validate_impl, query)

ggsql_version_impl <- function() .Call(wrap__ggsql_version_impl)

GgsqlReader <- new.env(parent = emptyenv())

GgsqlReader$new <- function(connection) .Call(wrap__GgsqlReader__new, connection)

GgsqlReader$new_custom <- function(execute_sql, register, unregister) .Call(wrap__GgsqlReader__new_custom, execute_sql, register, unregister)

GgsqlReader$register_ipc <- function(name, ipc_bytes, replace) invisible(.Call(wrap__GgsqlReader__register_ipc, self, name, ipc_bytes, replace))

GgsqlReader$unregister <- function(name) invisible(.Call(wrap__GgsqlReader__unregister, self, name))

GgsqlReader$execute_sql_ipc <- function(sql) .Call(wrap__GgsqlReader__execute_sql_ipc, self, sql)

GgsqlReader$execute <- function(query) .Call(wrap__GgsqlReader__execute, self, query)

#' @export
`$.GgsqlReader` <- function (self, name) { func <- GgsqlReader[[name]]; environment(func) <- environment(); func }

#' @export
`[[.GgsqlReader` <- `$.GgsqlReader`

GgsqlSpec <- new.env(parent = emptyenv())

GgsqlSpec$metadata_rows <- function() .Call(wrap__GgsqlSpec__metadata_rows, self)

GgsqlSpec$metadata_columns <- function() .Call(wrap__GgsqlSpec__metadata_columns, self)

GgsqlSpec$metadata_layer_count <- function() .Call(wrap__GgsqlSpec__metadata_layer_count, self)

GgsqlSpec$get_sql <- function() .Call(wrap__GgsqlSpec__get_sql, self)

GgsqlSpec$get_visual <- function() .Call(wrap__GgsqlSpec__get_visual, self)

GgsqlSpec$layer_count <- function() .Call(wrap__GgsqlSpec__layer_count, self)

GgsqlSpec$layer_data_ipc <- function(index) .Call(wrap__GgsqlSpec__layer_data_ipc, self, index)

GgsqlSpec$stat_data_ipc <- function(index) .Call(wrap__GgsqlSpec__stat_data_ipc, self, index)

GgsqlSpec$get_layer_sql <- function(index) .Call(wrap__GgsqlSpec__get_layer_sql, self, index)

GgsqlSpec$get_stat_sql <- function(index) .Call(wrap__GgsqlSpec__get_stat_sql, self, index)

GgsqlSpec$warnings_json <- function() .Call(wrap__GgsqlSpec__warnings_json, self)

#' @export
`$.GgsqlSpec` <- function (self, name) { func <- GgsqlSpec[[name]]; environment(func) <- environment(); func }

#' @export
`[[.GgsqlSpec` <- `$.GgsqlSpec`

GgsqlWriter <- new.env(parent = emptyenv())

GgsqlWriter$new <- function() .Call(wrap__GgsqlWriter__new)

GgsqlWriter$render <- function(spec) .Call(wrap__GgsqlWriter__render, self, spec)

#' @export
`$.GgsqlWriter` <- function (self, name) { func <- GgsqlWriter[[name]]; environment(func) <- environment(); func }

#' @export
`[[.GgsqlWriter` <- `$.GgsqlWriter`


# nolint end

check_custom <- function(
  x,
  test,
  expected,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x) && ((allow_null && is.null(x)) || test(x))) {
    return(invisible(NULL))
  }

  stop_input_type(
    x,
    expected,
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_r6 <- function(
  x,
  class,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  check_custom(
    x,
    function(x) R6::is.R6(x) && inherits(x, class),
    paste0("a ", class, "/R6 object")
  )
}

#' Convert a data.frame to Arrow IPC stream bytes
#' @param df A data.frame.
#' @return Raw vector of Arrow IPC stream bytes.
#' @noRd
df_to_ipc <- function(df) {
  # Convert factors to character — nanoarrow doesn't support dictionary IPC encoding
  factor_cols <- vapply(df, is.factor, logical(1))
  if (any(factor_cols)) {
    df[factor_cols] <- lapply(df[factor_cols], as.character)
  }
  stream <- nanoarrow::as_nanoarrow_array_stream(df)
  con <- rawConnection(raw(0), "wb")
  on.exit(close(con))
  nanoarrow::write_nanoarrow(stream, con)
  rawConnectionValue(con)
}

#' Convert Arrow IPC stream bytes to a data.frame
#' @param ipc_bytes Raw vector of Arrow IPC stream bytes.
#' @return A data.frame.
#' @noRd
ipc_to_df <- function(ipc_bytes) {
  con <- rawConnection(ipc_bytes, "rb")
  on.exit(close(con))
  stream <- nanoarrow::read_nanoarrow(con)
  as.data.frame(stream)
}

#' Create a DuckDB reader
#'
#' Creates a DuckDB database connection that can execute SQL queries and
#' register data frames as queryable tables. The default creates an empty
#' in-memory database but you can also pass the path to a DuckDB database to
#' directly interact with that.
#'
#' @param database Path to a DuckDB database file, or `NULL` (the default)
#' for an in-memory database.
#'
#' @return A `Reader` object.
#'
#' @export
#'
#' @family readers
#'
#' @examples
#' reader <- duckdb_reader()
#' ggsql_register(reader, mtcars, "cars")
#' df <- ggsql_execute_sql(reader, "SELECT mpg, disp FROM cars LIMIT 5")
#'
duckdb_reader <- function(database = NULL) {
  check_string(database, allow_empty = FALSE, allow_null = TRUE)
  if (is.null(database)) {
    connection <- "duckdb://memory"
  } else {
    if (!file.exists(database)) {
      cli::cli_abort("Database file {.path {database}} does not exist.")
    }
    connection <- paste0("duckdb://", normalizePath(database))
  }
  Reader$new(connection)
}

#' Create an ODBC reader
#'
#' Creates a connection to a database through an ODBC driver. Can execute SQL
#' queries and, where the backend supports it, register R data frames as
#' temporary tables. Requires an ODBC driver manager (unixODBC/iODBC on Unix,
#' built into Windows) and the appropriate ODBC driver for the target database
#' to be installed.
#'
#' Either pass a full ODBC connection string via `connection_string`, or supply
#' named components (`dsn`, `driver`, `server`, `database`, `uid`, `pwd`, plus
#' any extra `key = value` pairs in `...`) and they will be assembled into a
#' connection string. If `connection_string` is supplied, the other named
#' arguments are ignored.
#'
#' @param dsn Name of a data source configured in `odbc.ini` / `odbcinst.ini`.
#' @param driver ODBC driver name (e.g. `"{PostgreSQL}"`, `"{Snowflake}"`).
#' Curly brackets are optional.
#' @param server,database,uid,pwd Common ODBC parameters.
#' @param ... Additional named `key = value` parameters appended to the
#'   connection string (e.g. `Port = 5432`, `Warehouse = "COMPUTE_WH"`).
#' @param connection_string A full ODBC connection string, e.g.
#'   `"Driver={PostgreSQL};Server=localhost;Database=mydb;UID=user;PWD=pass"`.
#'   A leading `odbc://` is accepted and stripped.
#'
#' @return A `Reader` object.
#'
#' @section Credentials:
#' Connection strings are stored in-memory for the life of the reader. Prefer
#' configuring credentials through a DSN, `~/.odbc.ini`, or environment
#' variables rather than hard-coding passwords in scripts.
#'
#' @export
#'
#' @family readers
#'
#' @examples
#' \dontrun{
#' # Using a preconfigured DSN
#' reader <- odbc_reader(dsn = "mydsn")
#'
#' # Building a connection string from components
#' reader <- odbc_reader(
#'   driver = "{PostgreSQL}",
#'   server = "localhost",
#'   database = "mydb",
#'   uid = "user",
#'   pwd = "secret",
#'   Port = 5432
#' )
#'
#' # Passing a full connection string
#' reader <- odbc_reader("Driver={SQLite3};Database=:memory:")
#' }
#'
odbc_reader <- function(
  dsn = NULL,
  driver = NULL,
  server = NULL,
  database = NULL,
  uid = NULL,
  pwd = NULL,
  ...,
  connection_string = NULL
) {
  uri <- build_odbc_uri(
    connection_string = connection_string,
    dsn = dsn,
    driver = driver,
    server = server,
    database = database,
    uid = uid,
    pwd = pwd,
    extras = list(...)
  )
  Reader$new(uri)
}

build_odbc_uri <- function(
  connection_string = NULL,
  dsn = NULL,
  driver = NULL,
  server = NULL,
  database = NULL,
  uid = NULL,
  pwd = NULL,
  extras = list()
) {
  check_string(connection_string, allow_empty = FALSE, allow_null = TRUE)
  if (!is.null(connection_string)) {
    conn <- sub("^odbc://", "", connection_string, ignore.case = TRUE)
    return(paste0("odbc://", conn))
  }
  parts <- c(
    if (!is.null(dsn)) paste0("DSN=", dsn),
    if (!is.null(driver)) {
      paste0("Driver={", gsub("^\\{|\\}$", "", driver), "}")
    },
    if (!is.null(server)) paste0("Server=", server),
    if (!is.null(database)) paste0("Database=", database),
    if (!is.null(uid)) paste0("UID=", uid),
    if (!is.null(pwd)) paste0("PWD=", pwd)
  )
  if (length(extras) > 0) {
    nm <- names(extras)
    if (is.null(nm) || !all(nzchar(nm))) {
      cli::cli_abort(
        "All {.arg ...} arguments to {.fn odbc_reader} must be named."
      )
    }
    parts <- c(
      parts,
      paste0(nm, "=", vapply(extras, as.character, character(1)))
    )
  }
  if (length(parts) == 0) {
    cli::cli_abort(
      "Must supply {.arg connection_string} or at least one named ODBC parameter."
    )
  }
  paste0("odbc://", paste(parts, collapse = ";"))
}

#' Create a Snowflake reader
#'
#' Convenience constructor for Snowflake connections. Uses the ODBC reader
#' under the hood with the `Snowflake` driver, and takes advantage of the
#' dedicated Snowflake handling in the ggsql Rust core:
#'
#' Special handling of Snowflake includes:
#'
#' - If `connection_name` is supplied (or `ConnectionName=` appears in the
#'   connection string), it is resolved against `~/.snowflake/connections.toml`.
#' - When running inside Posit Workbench, an OAuth token is automatically
#'   injected if one is available.
#' - Schema introspection uses Snowflake's `SHOW DATABASES / SCHEMAS / TABLES`
#'   commands rather than `information_schema`.
#'
#' Requires the Snowflake ODBC driver to be installed on the system.
#'
#' @param account Snowflake account identifier (e.g. `"xy12345"` or
#'   `"xy12345.us-east-1"`). Translated to
#'   `Server={account}.snowflakecomputing.com` in the connection string.
#' @param warehouse,database,schema,role Snowflake session defaults.
#' @param user,password User credentials. Prefer a DSN, `connection_name`, or
#'   `authenticator = "externalbrowser"` over hard-coded passwords.
#' @param authenticator Snowflake authenticator (e.g. `"externalbrowser"`,
#'   `"snowflake_jwt"`, `"oauth"`).
#' @param connection_name Named entry in `~/.snowflake/connections.toml` whose
#'   fields will fill in the remaining connection parameters.
#' @param driver Override the ODBC driver name (defaults to `"Snowflake"`).
#' @param ... Additional named `key = value` parameters appended to the
#'   connection string.
#' @param connection_string A full raw connection string, bypassing the named
#'   arguments. `Driver={Snowflake};` is prepended if it isn't already present.
#'
#' @return A `Reader` object.
#'
#' @export
#'
#' @family readers
#'
#' @examples
#' \dontrun{
#' # Using a named connection from ~/.snowflake/connections.toml
#' reader <- snowflake_reader(connection_name = "my_workbench")
#'
#' # Browser-based SSO
#' reader <- snowflake_reader(
#'   account = "xy12345.us-east-1",
#'   user = "alice@example.com",
#'   authenticator = "externalbrowser",
#'   warehouse = "COMPUTE_WH",
#'   database = "ANALYTICS",
#'   schema = "PUBLIC",
#'   role = "ANALYST"
#' )
#' }
#'
snowflake_reader <- function(
  account = NULL,
  warehouse = NULL,
  database = NULL,
  schema = NULL,
  role = NULL,
  user = NULL,
  password = NULL,
  authenticator = NULL,
  connection_name = NULL,
  driver = NULL,
  ...,
  connection_string = NULL
) {
  uri <- build_snowflake_uri(
    connection_string = connection_string,
    account = account,
    warehouse = warehouse,
    database = database,
    schema = schema,
    role = role,
    user = user,
    password = password,
    authenticator = authenticator,
    connection_name = connection_name,
    driver = driver,
    extras = list(...)
  )
  Reader$new(uri)
}

build_snowflake_uri <- function(
  connection_string = NULL,
  account = NULL,
  warehouse = NULL,
  database = NULL,
  schema = NULL,
  role = NULL,
  user = NULL,
  password = NULL,
  authenticator = NULL,
  connection_name = NULL,
  driver = NULL,
  extras = list()
) {
  check_string(connection_string, allow_empty = FALSE, allow_null = TRUE)
  if (!is.null(connection_string)) {
    conn <- sub("^snowflake://", "", connection_string, ignore.case = TRUE)
    return(paste0("snowflake://", conn))
  }
  server <- if (!is.null(account)) {
    if (grepl("\\.", account, fixed = FALSE)) {
      # account already includes a region / full host suffix
      if (endsWith(tolower(account), ".snowflakecomputing.com")) {
        account
      } else {
        paste0(account, ".snowflakecomputing.com")
      }
    } else {
      paste0(account, ".snowflakecomputing.com")
    }
  }
  parts <- c(
    if (!is.null(driver)) paste0("Driver=", driver),
    if (!is.null(connection_name)) paste0("ConnectionName=", connection_name),
    if (!is.null(server)) paste0("Server=", server),
    if (!is.null(warehouse)) paste0("Warehouse=", warehouse),
    if (!is.null(database)) paste0("Database=", database),
    if (!is.null(schema)) paste0("Schema=", schema),
    if (!is.null(role)) paste0("Role=", role),
    if (!is.null(user)) paste0("UID=", user),
    if (!is.null(password)) paste0("PWD=", password),
    if (!is.null(authenticator)) paste0("Authenticator=", authenticator)
  )
  if (length(extras) > 0) {
    nm <- names(extras)
    if (is.null(nm) || !all(nzchar(nm))) {
      cli::cli_abort(
        "All {.arg ...} arguments to {.fn snowflake_reader} must be named."
      )
    }
    parts <- c(
      parts,
      paste0(nm, "=", vapply(extras, as.character, character(1)))
    )
  }
  if (length(parts) == 0) {
    cli::cli_abort(
      "Must supply {.arg connection_string}, {.arg connection_name}, or at least one connection parameter."
    )
  }
  paste0("snowflake://", paste(parts, collapse = ";"))
}

#' Create a reader backed by R callbacks
#'
#' Construct a reader whose behavior is defined entirely by R functions
#' you supply. This makes it possible to plug in data sources that aren't
#' provided natively by ggsql (e.g. an in-memory store, a custom HTTP API,
#' a DBI connection, etc.) without touching the Rust side.
#'
#' Only `execute_sql` is required. If `register` or `unregister` are
#' omitted, calling [ggsql_register()] / [ggsql_unregister()] on the
#' returned reader raises an error.
#'
#' @param execute_sql A function `function(sql)` that executes `sql` and
#'   returns either a data frame or a raw vector containing Arrow IPC
#'   stream bytes (as produced by `nanoarrow::as_nanoarrow_array_stream()`
#'   / arrow IPC writers).
#' @param register Optional `function(name, df, replace)` that registers
#'   `df` as a table named `name`. `replace` is `TRUE` if the caller
#'   expects an existing table with the same name to be replaced.
#' @param unregister Optional `function(name)` that removes a previously
#'   registered table.
#'
#' @return A `Reader` object, usable anywhere the other `*_reader()`
#'   constructors are accepted.
#'
#' @export
#'
#' @family readers
#'
#' @examples
#' # A trivial reader backed by a list of data frames in an environment,
#' # delegating the actual SQL engine to an in-memory DuckDB.
#' store <- new.env(parent = emptyenv())
#' backend <- duckdb_reader()
#' reader <- custom_reader(
#'   execute_sql = function(sql) ggsql_execute_sql(backend, sql),
#'   register = function(name, df, replace) {
#'     store[[name]] <- df
#'     ggsql_register(backend, df, name, replace = replace)
#'   },
#'   unregister = function(name) {
#'     rm(list = name, envir = store)
#'     ggsql_unregister(backend, name)
#'   }
#' )
#' ggsql_register(reader, mtcars, "cars")
#' ggsql_execute_sql(reader, "SELECT mpg, disp FROM cars LIMIT 3")
#'
custom_reader <- function(execute_sql, register = NULL, unregister = NULL) {
  check_function(execute_sql)
  check_function(register, allow_null = TRUE)
  check_function(unregister, allow_null = TRUE)

  # The Rust side always exchanges tables as Arrow IPC bytes. We wrap the
  # user's hooks so Rust never sees an R data.frame. NULL maps to an empty
  # frame — matches the public `ggsql_execute_sql()` contract for value-less
  # statements (e.g. CREATE TABLE) that ggsql issues internally.
  exec_wrapped <- function(sql) {
    out <- execute_sql(sql)
    if (is.null(out)) out <- data.frame()
    if (is.raw(out)) out else df_to_ipc(out)
  }
  reg_wrapped <- if (!is.null(register)) {
    function(name, ipc_bytes, replace) {
      register(name, ipc_to_df(ipc_bytes), isTRUE(replace))
      invisible(NULL)
    }
  }

  reader <- Reader$new(NULL)
  reader$.ptr <- GgsqlReader$new_custom(exec_wrapped, reg_wrapped, unregister)
  reader
}

# DuckDB writes `~/.duckdb/...` on demand for extension autoload, persistent
# secrets, etc. CRAN forbids writing under the user's home directory, so
# redirect those locations into the R session temp dir. `extension_directory`
# and `secret_directory` are resolved once from `home_directory` at startup,
# so a runtime `SET home_directory` alone is too late — they must be set
# explicitly. If the user already has `~/.duckdb` we leave it alone: they
# likely have extensions cached from other DuckDB use and would have to
# re-download every R session if redirected. CRAN check machines never have
# this directory, so the redirect kicks in there.
configure_duckdb_paths <- function(reader_ptr) {
  if (dir.exists(path.expand("~/.duckdb"))) {
    return(invisible(NULL))
  }
  base <- file.path(tempdir(), ".duckdb")
  base <- normalizePath(base, winslash = "/", mustWork = FALSE)
  base <- gsub("'", "''", base, fixed = TRUE)
  reader_ptr$execute_sql_ipc(sprintf("SET home_directory = '%s'", base))
  reader_ptr$execute_sql_ipc(
    sprintf("SET extension_directory = '%s/extensions'", base)
  )
  reader_ptr$execute_sql_ipc(
    sprintf("SET secret_directory = '%s/secrets'", base)
  )
  invisible(NULL)
}

#' @noRd
Reader <- R6::R6Class(
  "Reader",
  cloneable = FALSE,
  public = list(
    .ptr = NULL,

    initialize = function(connection) {
      # `NULL` lets subclasses / factory constructors install a custom
      # `.ptr` themselves (see `custom_reader()`).
      if (!is.null(connection)) {
        self$.ptr <- GgsqlReader$new(connection)
        if (startsWith(tolower(connection), "duckdb://")) {
          configure_duckdb_paths(self$.ptr)
        }
      }
    },

    print = function(...) {
      cli::cli_text("<ggsql_reader>")
      invisible(self)
    }
  )
)

#' Register and unregisters a data frame as a queryable table
#'
#' After registration, the data frame can be queried by name in SQL statements.
#' You can use `ggsql_table` to extract tables from the reader (both registered
#' ones and those native to the backend) and `ggsql_table_names` to get a vector
#' of all the tables in reader.
#'
#' @param reader A `Reader` object created by e.g. [duckdb_reader()].
#' @param df A data frame to register.
#' @param name The name of the table.
#' @param replace If `TRUE`, replace an existing table with the same name.
#' Defaults to `FALSE`.
#'
#' @return `reader` for `ggsql_register()` and `ggsql_unregister()`.
#' `ggsql_table` returns a data.frame if the table exists and `NULL` if not.
#' `ggsql_table_names` return a character vector.
#'
#' @export
#'
#' @examples
#' reader <- duckdb_reader()
#' ggsql_register(reader, mtcars, "cars")
#'
#' ggsql_table_names(reader)
#'
#' ggsql_table(reader, "cars")
#'
#' ggsql_unregister(reader, "cars")
#'
#' ggsql_table_names(reader)
#'
ggsql_register <- function(reader, df, name, replace = FALSE) {
  check_r6(reader, "Reader")
  check_data_frame(df)
  check_string(name, allow_empty = FALSE)
  ipc_bytes <- df_to_ipc(df)
  reader$.ptr$register_ipc(name, ipc_bytes, replace)
  invisible(reader)
}

#' @rdname ggsql_register
#' @export
ggsql_unregister <- function(reader, name) {
  check_r6(reader, "Reader")
  check_string(name, allow_empty = FALSE)
  reader$.ptr$unregister(name)
  invisible(reader)
}

#' @rdname ggsql_register
#' @export
ggsql_table <- function(reader, name) {
  if (name %in% ggsql_table_names(reader)) {
    safe_name <- gsub('"', '""', name, fixed = TRUE)
    ggsql_execute_sql(reader, paste0("SELECT * FROM \"", safe_name, "\""))
  } else {
    NULL
  }
}

#' @rdname ggsql_register
#' @export
ggsql_table_names <- function(reader) {
  ggsql_execute_sql(reader, "SHOW TABLES")$name
}

#' Execute a ggsql query
#'
#' Parses the query, and execute it against the reader's database. Returns
#' either a visualization specification ready for rendering (`ggsql_execute`) or
#' a data frame with the query result (`ggsql_execute_sql`).
#'
#' @param reader A `Reader` object created by e.g. [duckdb_reader()] or
#' [odbc_reader()].
#' @param query A ggsql query string (SQL + VISUALISE clause).
#'
#' @return `ggsql_execute()` returns `Spec` object. `ggsql_execute_sql()`
#' returns a data frame or `NULL`
#'
#' @export
#'
#' @examples
#' reader <- duckdb_reader()
#' ggsql_register(reader, mtcars, "cars")
#' spec <- ggsql_execute(reader,
#'   "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
#' )
#'
ggsql_execute <- function(reader, query) {
  check_r6(reader, "Reader")
  check_string(query, allow_empty = FALSE)
  spec_ptr <- reader$.ptr$execute(query)
  Spec$new(spec_ptr)
}

#' @rdname ggsql_execute
#' @export
ggsql_execute_sql <- function(reader, query) {
  check_r6(reader, "Reader")
  check_string(query, allow_empty = FALSE)
  ipc_bytes <- reader$.ptr$execute_sql_ipc(query)
  df <- ipc_to_df(ipc_bytes)
  # DDL/DML statements (CREATE, INSERT, COPY, ...) come back as a 0x0 frame.
  # Surface that as NULL so callers can branch on `is.null()` instead of
  # poking at dim().
  if (ncol(df) == 0L) {
    return(invisible(NULL))
  }
  df
}

test_that("vegalite_writer creates a writer", {
  writer <- vegalite_writer()
  expect_s3_class(writer, "Writer")
})

test_that("ggsql_render returns Vega-Lite JSON", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  writer <- vegalite_writer()
  json <- ggsql_render(writer, spec)
  expect_type(json, "character")
  expect_match(json, "vega-lite")
  # Should be valid JSON
  parsed <- jsonlite::fromJSON(json)
  expect_true("$schema" %in% names(parsed))
})

test_that("duckdb_reader creates a reader", {
  reader <- duckdb_reader()
  expect_s3_class(reader, "Reader")
})

test_that("ggsql_register registers a data frame", {
  reader <- duckdb_reader()
  result <- ggsql_register(reader, mtcars, "cars")
  expect_invisible(ggsql_register(reader, iris, "iris"))
  # Returns reader for piping

  expect_s3_class(result, "Reader")
})

test_that("ggsql_register with replace works", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "data")
  ggsql_register(reader, iris, "data", replace = TRUE)
  df <- ggsql_execute_sql(reader, "SELECT * FROM data LIMIT 1")
  expect_true("Sepal.Length" %in% names(df))
})

test_that("ggsql_unregister removes a table", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  ggsql_unregister(reader, "cars")
  expect_error(ggsql_execute_sql(reader, "SELECT * FROM cars"))
})

test_that("ggsql_execute_sql returns a data frame", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  df <- ggsql_execute_sql(reader, "SELECT mpg, disp FROM cars LIMIT 5")
  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 5)
  expect_equal(names(df), c("mpg", "disp"))
})

test_that("ggsql_execute_sql returns NULL for value-less statements", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  expect_null(ggsql_execute_sql(reader, "CREATE VIEW v AS SELECT mpg FROM cars"))
  expect_null(ggsql_execute_sql(
    reader,
    "CREATE TABLE t AS SELECT * FROM cars LIMIT 3"
  ))
  expect_null(ggsql_execute_sql(reader, "INSERT INTO t SELECT * FROM cars LIMIT 1"))
  expect_null(ggsql_execute_sql(reader, "DROP VIEW v"))

  # Empty result sets keep their schema — they're a real value, just zero rows.
  df <- ggsql_execute_sql(reader, "SELECT mpg FROM cars WHERE 1 = 0")
  expect_s3_class(df, "data.frame")
  expect_equal(names(df), "mpg")
  expect_equal(nrow(df), 0)
})

test_that("ggsql_execute returns a spec", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  expect_s3_class(spec, "Spec")
})

test_that("piping works", {
  reader <- duckdb_reader()
  result <- reader |>
    ggsql_register(mtcars, "cars") |>
    ggsql_execute_sql("SELECT mpg FROM cars LIMIT 3")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3)
})

# --- odbc_reader / build_odbc_uri ---

test_that("build_odbc_uri assembles a URI from named parts", {
  uri <- build_odbc_uri(
    driver = "{PostgreSQL}",
    server = "localhost",
    database = "mydb",
    uid = "user",
    pwd = "secret"
  )
  expect_equal(
    uri,
    "odbc://Driver={PostgreSQL};Server=localhost;Database=mydb;UID=user;PWD=secret"
  )
})

test_that("build_odbc_uri accepts a DSN", {
  expect_equal(build_odbc_uri(dsn = "mydsn"), "odbc://DSN=mydsn")
})

test_that("build_odbc_uri appends ... parameters as key=value pairs", {
  uri <- build_odbc_uri(
    dsn = "mydsn",
    extras = list(Port = 5432, Warehouse = "COMPUTE_WH")
  )
  expect_equal(uri, "odbc://DSN=mydsn;Port=5432;Warehouse=COMPUTE_WH")
})

test_that("build_odbc_uri preserves a raw connection string", {
  expect_equal(
    build_odbc_uri(connection_string = "Driver={SQLite3};Database=:memory:"),
    "odbc://Driver={SQLite3};Database=:memory:"
  )
})

test_that("build_odbc_uri strips a leading odbc:// from the raw string", {
  expect_equal(
    build_odbc_uri(connection_string = "odbc://DSN=mydsn"),
    "odbc://DSN=mydsn"
  )
})

test_that("build_odbc_uri prefers connection_string over named parts", {
  expect_equal(
    build_odbc_uri(
      connection_string = "DSN=direct",
      driver = "{PostgreSQL}",
      server = "ignored"
    ),
    "odbc://DSN=direct"
  )
})

test_that("build_odbc_uri errors when nothing is supplied", {
  expect_error(build_odbc_uri(), "at least one named ODBC parameter")
})

test_that("build_odbc_uri errors when extras are unnamed", {
  expect_error(
    build_odbc_uri(dsn = "mydsn", extras = list("unnamed")),
    "must be named"
  )
})

test_that("odbc_reader errors when called with no args", {
  expect_error(odbc_reader(), "at least one named ODBC parameter")
})

# --- snowflake_reader / build_snowflake_uri ---

test_that("build_snowflake_uri assembles Server from account", {
  uri <- build_snowflake_uri(
    account = "xy12345",
    user = "alice",
    password = "s3cret",
    warehouse = "COMPUTE_WH"
  )
  expect_equal(
    uri,
    "snowflake://Server=xy12345.snowflakecomputing.com;Warehouse=COMPUTE_WH;UID=alice;PWD=s3cret"
  )
})

test_that("build_snowflake_uri preserves full account hostnames", {
  uri <- build_snowflake_uri(account = "xy12345.us-east-1")
  expect_equal(
    uri,
    "snowflake://Server=xy12345.us-east-1.snowflakecomputing.com"
  )
})

test_that("build_snowflake_uri does not double the .snowflakecomputing.com suffix", {
  uri <- build_snowflake_uri(
    account = "xy12345.us-east-1.snowflakecomputing.com"
  )
  expect_equal(
    uri,
    "snowflake://Server=xy12345.us-east-1.snowflakecomputing.com"
  )
})

test_that("build_snowflake_uri emits ConnectionName alone", {
  expect_equal(
    build_snowflake_uri(connection_name = "my_workbench"),
    "snowflake://ConnectionName=my_workbench"
  )
})

test_that("build_snowflake_uri respects a driver override", {
  uri <- build_snowflake_uri(
    driver = "{SnowflakeDSIIDriver}",
    account = "xy12345"
  )
  expect_equal(
    uri,
    "snowflake://Driver={SnowflakeDSIIDriver};Server=xy12345.snowflakecomputing.com"
  )
})

test_that("build_snowflake_uri accepts extras", {
  uri <- build_snowflake_uri(
    connection_name = "my_wb",
    extras = list(Tracing = 0, Application = "ggsql")
  )
  expect_equal(
    uri,
    "snowflake://ConnectionName=my_wb;Tracing=0;Application=ggsql"
  )
})

test_that("build_snowflake_uri errors on unnamed extras", {
  expect_error(
    build_snowflake_uri(account = "xy12345", extras = list("unnamed")),
    "must be named"
  )
})

test_that("build_snowflake_uri errors when nothing is supplied", {
  expect_error(build_snowflake_uri(), "at least one connection parameter")
})

test_that("build_snowflake_uri passes a raw connection string through", {
  expect_equal(
    build_snowflake_uri(
      connection_string = "Driver=Snowflake;Server=xy12345.snowflakecomputing.com"
    ),
    "snowflake://Driver=Snowflake;Server=xy12345.snowflakecomputing.com"
  )
})

test_that("snowflake_reader errors with no args", {
  expect_error(snowflake_reader(), "at least one connection parameter")
})

# --- custom_reader ---

# Helper: build a custom reader that delegates everything to an in-memory
# DuckDB. This exercises all three callback paths end-to-end.
delegating_custom_reader <- function(log = NULL) {
  backend <- duckdb_reader()
  record <- function(hook) {
    if (is.null(log)) {
      return(invisible())
    }
    prev <- if (is.null(log[[hook]])) 0L else log[[hook]]
    log[[hook]] <- prev + 1L
  }
  custom_reader(
    execute_sql = function(sql) {
      record("execute_sql")
      ggsql_execute_sql(backend, sql)
    },
    register = function(name, df, replace) {
      record("register")
      ggsql_register(backend, df, name, replace = replace)
    },
    unregister = function(name) {
      record("unregister")
      ggsql_unregister(backend, name)
    }
  )
}

test_that("custom_reader returns a Reader", {
  reader <- delegating_custom_reader()
  expect_s3_class(reader, "Reader")
})

test_that("custom_reader dispatches register / execute_sql / unregister", {
  log <- new.env(parent = emptyenv())
  reader <- delegating_custom_reader(log)

  ggsql_register(reader, mtcars, "cars")
  df <- ggsql_execute_sql(reader, "SELECT mpg, cyl FROM cars LIMIT 4")
  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 4)
  expect_equal(names(df), c("mpg", "cyl"))

  ggsql_unregister(reader, "cars")
  expect_equal(log$register, 1L)
  expect_true(log$execute_sql >= 1L)
  expect_equal(log$unregister, 1L)
})

test_that("custom_reader works with a VISUALISE query", {
  reader <- delegating_custom_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  expect_s3_class(spec, "Spec")
})

test_that("custom_reader accepts execute_sql returning raw IPC bytes", {
  # Bypass the df_to_ipc() wrapper by returning IPC bytes directly.
  backend <- duckdb_reader()
  ggsql_register(backend, mtcars, "cars")
  reader <- custom_reader(
    execute_sql = function(sql) {
      df <- ggsql_execute_sql(backend, sql)
      df_to_ipc(df)
    }
  )
  df <- ggsql_execute_sql(reader, "SELECT mpg FROM cars LIMIT 2")
  expect_equal(nrow(df), 2)
})

test_that("custom_reader errors on register when no register hook given", {
  reader <- custom_reader(
    execute_sql = function(sql) data.frame(x = 1:3)
  )
  expect_error(ggsql_register(reader, mtcars, "cars"))
})

test_that("custom_reader errors on unregister when no unregister hook given", {
  reader <- custom_reader(
    execute_sql = function(sql) data.frame(x = 1:3),
    register = function(name, df, replace) invisible(NULL)
  )
  expect_error(ggsql_unregister(reader, "anything"))
})

test_that("custom_reader validates its arguments", {
  expect_error(custom_reader("not a function"), "must be a function")
  expect_error(
    custom_reader(function(sql) NULL, register = "nope"),
    "must be a function"
  )
  expect_error(
    custom_reader(function(sql) NULL, unregister = 42),
    "must be a function"
  )
})

run_query <- function(query, ...) {
  opts <- knitr::opts_current$get()
  opts$code <- query
  opts$screenshot.force <- FALSE
  extra_opts <- list(...)
  if (length(extra_opts) > 0) {
    opts[names(extra_opts)] <- extra_opts
  }
  ggsql_engine(opts)
}

data_file <- "mtcars.csv"
on.exit(unlink(data_file))
write.csv(mtcars, data_file)

test_that("engine can handle a query", {
  query <- c(
    paste0("SELECT mpg, disp FROM '", data_file, "'"),
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )
  out <- run_query(query, dev = "png")

  # We expect path to png file here, since output format for knitr is undetermined
  expect_type(out, "character")
  expect_length(out, 1L)
})

test_that("interactive writer keeps the htmlwidget path at narrow figure widths", {
  query <- c(
    paste0("SELECT mpg, disp FROM '", data_file, "'"),
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )

  out <- run_query(query, fig.width = 2.35, fig.height = 4)

  expect_match(out, "ggsql_vega")
  expect_match(out, "<ggsql-vega", fixed = TRUE)
})

test_that("engine can handle a query without visualisation statement", {
  query <- paste0("SELECT mpg, disp FROM '", data_file, "'")

  out <- run_query(query)
  expect_snapshot(cat(out))
})

test_that("engine does not return a table when merely creating data", {
  skip_if_not_installed("withr")
  tmp <- withr::local_tempdir()
  withr::local_dir(tmp)

  query <-
    "COPY (
      SELECT * FROM (VALUES
          (5.2, 18.5),
          (8.7, 22.3)
      ) AS t(x, y)
    ) TO 'data.csv' (HEADER, DELIMITER ',')"
  out <- run_query(query)
  expect_snapshot(cat(out))
})

# --- Data reference tests (r: and py: prefixes) ---

test_that("r: prefix resolves R objects", {
  # Put data into the knitr global environment
  assign("test_df", mtcars[1:5, c("mpg", "disp")], envir = knitr::knit_global())
  on.exit(rm("test_df", envir = knitr::knit_global()))

  query <- "SELECT mpg, disp FROM r:test_df"
  out <- run_query(query)
  expect_snapshot(cat(out))
})

test_that("r: prefix works in visualisation queries", {
  assign("test_df", mtcars[1:5, c("mpg", "disp")], envir = knitr::knit_global())
  on.exit(rm("test_df", envir = knitr::knit_global()))

  query <- c(
    "SELECT * FROM r:test_df",
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )
  out <- run_query(query, dev = "png")
  expect_type(out, "character")
  expect_length(out, 1L)
})

test_that("r: prefix errors for missing objects", {
  query <- "SELECT * FROM r:nonexistent_object_xyz"
  out <- run_query(query)
  expect_match(out, "not found", ignore.case = TRUE)
})

test_that("r: prefix errors for non-data-frame objects", {
  assign("not_a_df", "just a string", envir = knitr::knit_global())
  on.exit(rm("not_a_df", envir = knitr::knit_global()))

  query <- "SELECT * FROM r:not_a_df"
  out <- run_query(query)
  expect_match(out, "data frame", ignore.case = TRUE)
})

test_that("multiple r: refs in one query work", {
  assign("df_a", data.frame(id = 1:3, x = 10:12), envir = knitr::knit_global())
  assign("df_b", data.frame(id = 1:3, y = 20:22), envir = knitr::knit_global())
  on.exit({
    rm("df_a", envir = knitr::knit_global())
    rm("df_b", envir = knitr::knit_global())
  })

  query <- "SELECT a.id, a.x, b.y FROM r:df_a a JOIN r:df_b b ON a.id = b.id"
  out <- run_query(query)
  expect_snapshot(cat(out))
})

# --- output.var and sql proxy tests ---

test_that("output.var captures SQL result as data frame", {
  query <- "SELECT 1 AS x, 2 AS y"
  run_query(query, output.var = "captured_df")
  df <- get("captured_df", envir = knitr::knit_global())
  on.exit(rm("captured_df", envir = knitr::knit_global()))

  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 1)
  expect_equal(names(df), c("x", "y"))
})

test_that("output.var captures Vega-Lite JSON for viz queries", {
  assign("test_df", mtcars[1:5, c("mpg", "disp")], envir = knitr::knit_global())
  on.exit(
    rm(list = c("test_df", "captured_json"), envir = knitr::knit_global()),
    add = TRUE
  )

  query <- c(
    "SELECT * FROM r:test_df",
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )
  run_query(query, output.var = "captured_json")
  json <- get("captured_json", envir = knitr::knit_global())

  expect_type(json, "character")
  expect_match(json, "vega-lite")
})

#test_that("sql proxy can access registered tables", {
#  reader <- get_engine_reader()
#  ggsql_register(reader, mtcars[1:3, c("mpg", "disp")], "proxy_test")
#
#  sql_obj <- get("sql", envir = knitr::knit_global())
#  df <- sql_obj$proxy_test
#  expect_s3_class(df, "data.frame")
#  expect_equal(nrow(df), 3)
#  expect_equal(names(df), c("mpg", "disp"))
#})

#test_that("sql proxy names() lists tables", {
#  reader <- get_engine_reader()
#  ggsql_register(reader, data.frame(a = 1), "names_test")
#
#  sql_obj <- get("sql", envir = knitr::knit_global())
#  tbl_names <- names(sql_obj)
#  expect_true("names_test" %in% tbl_names)
#})

# --- Inline chunk options (--| and #|) ---

test_that("--| prefix parses chunk options", {
  query <- c(
    "--| output.var: my_result",
    "SELECT 1 AS x, 2 AS y"
  )
  run_query(query)
  df <- get("my_result", envir = knitr::knit_global())
  on.exit(rm("my_result", envir = knitr::knit_global()))

  expect_s3_class(df, "data.frame")
  expect_equal(names(df), c("x", "y"))
})

test_that("#| prefix parses chunk options", {
  query <- c(
    "#| output.var: my_result2",
    "SELECT 1 AS a, 2 AS b"
  )
  run_query(query)
  df <- get("my_result2", envir = knitr::knit_global())
  on.exit(rm("my_result2", envir = knitr::knit_global()))

  expect_s3_class(df, "data.frame")
  expect_equal(names(df), c("a", "b"))
})

test_that("Quarto-style kebab-case options are converted", {
  query <- c(
    "--| output-var: my_result3",
    "SELECT 10 AS val"
  )
  run_query(query)
  df <- get("my_result3", envir = knitr::knit_global())
  on.exit(rm("my_result3", envir = knitr::knit_global()))

  expect_s3_class(df, "data.frame")
})

# --- Connection option tests ---

test_that("connection option creates a DuckDB reader", {
  query <- "SELECT 1 AS x, 2 AS y"
  out <- run_query(query, connection = "duckdb://memory")
  expect_snapshot(cat(out))
})

test_that("connection option rejects unsupported schemes", {
  query <- "SELECT 1 AS x"
  out <- run_query(query, connection = "mysql://localhost")
  expect_match(out, "Unsupported connection scheme")
  # The error should advertise all supported schemes
  expect_match(out, "duckdb")
  expect_match(out, "odbc")
  expect_match(out, "snowflake")
  expect_match(out, "custom")
})

test_that("connection option accepts odbc:// scheme", {
  # We can't connect without a real driver, but the scheme itself must be
  # accepted by parse_connection() — any failure should come from the driver
  # layer, not from the scheme parser.
  query <- "SELECT 1 AS x"
  out <- run_query(query, connection = "odbc://DSN=__ggsql_nonexistent__")
  expect_false(grepl("Unsupported connection scheme", out))
})

# --- custom:// connection ---

test_that("custom:// resolves a Reader from the knit environment", {
  log <- new.env(parent = emptyenv())
  backend <- duckdb_reader()
  ggsql_register(backend, mtcars[1:3, c("mpg", "cyl")], "cars")
  reader <- custom_reader(
    execute_sql = function(sql) {
      log$execute_sql <- TRUE
      ggsql_execute_sql(backend, sql)
    }
  )
  assign("my_reader", reader, envir = knitr::knit_global())
  on.exit(rm("my_reader", envir = knitr::knit_global()))

  out <- run_query(
    "SELECT mpg FROM cars LIMIT 2",
    connection = "custom://my_reader"
  )
  expect_true(isTRUE(log$execute_sql))
  expect_match(out, "mpg")
})

test_that("custom:// errors when the variable is missing", {
  out <- run_query(
    "SELECT 1",
    connection = "custom://__ggsql_no_such_reader__"
  )
  expect_match(out, "not found")
})

test_that("custom:// errors when the variable is not a Reader", {
  assign("not_a_reader", 42, envir = knitr::knit_global())
  on.exit(rm("not_a_reader", envir = knitr::knit_global()))

  out <- run_query("SELECT 1", connection = "custom://not_a_reader")
  expect_match(out, "Reader")
})

test_that("custom:// re-resolves on every chunk (no caching)", {
  count <- 0L
  make_reader <- function() {
    count <<- count + 1L
    backend <- duckdb_reader()
    custom_reader(
      execute_sql = function(sql) ggsql_execute_sql(backend, sql)
    )
  }
  assign("rebound_reader", make_reader(), envir = knitr::knit_global())
  on.exit(rm("rebound_reader", envir = knitr::knit_global()))

  run_query("SELECT 1 AS x", connection = "custom://rebound_reader")
  # Rebind to a fresh reader and confirm the engine picks it up
  assign("rebound_reader", make_reader(), envir = knitr::knit_global())
  run_query("SELECT 2 AS y", connection = "custom://rebound_reader")

  expect_equal(count, 2L)
})

test_that("connection option accepts snowflake:// scheme", {
  query <- "SELECT 1 AS x"
  out <- run_query(
    query,
    connection = "snowflake://ConnectionName=__ggsql_nonexistent__"
  )
  expect_false(grepl("Unsupported connection scheme", out))
})

test_that("connection option rejects invalid format", {
  query <- "SELECT 1 AS x"
  out <- run_query(query, connection = "not-a-uri")
  expect_match(out, "Invalid connection string")
})

# --- Writer option tests ---

test_that("writer defaults to interactive vegalite", {
  query <- c(
    paste0("SELECT mpg, disp FROM '", data_file, "'"),
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )
  out <- run_query(query)
  expect_match(out, "ggsql_vega")
})

test_that("writer = 'vegalite_svg' produces SVG output", {
  skip_if_not_installed("V8")
  skip_on_cran()
  skip_if_not_installed("withr")

  fig_dir <- withr::local_tempdir()

  query <- c(
    paste0("SELECT mpg, disp FROM '", data_file, "'"),
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )
  out <- run_query(
    query,
    writer = "vegalite_svg",
    fig.path = paste0(fig_dir, "/fig-"),
    label = "test-svg"
  )

  svg_files <- list.files(fig_dir, pattern = "\\.svg$")
  expect_length(svg_files, 1L)
  expect_match(out, "\\.svg")
})

test_that("writer = 'vegalite_png' produces PNG output", {
  skip_if_not_installed("V8")
  skip_if_not_installed("rsvg")
  skip_on_cran()
  skip_if_not_installed("withr")

  fig_dir <- withr::local_tempdir()

  query <- c(
    paste0("SELECT mpg, disp FROM '", data_file, "'"),
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )
  out <- run_query(
    query,
    writer = "vegalite_png",
    fig.path = paste0(fig_dir, "/fig-"),
    label = "test-png"
  )

  png_files <- list.files(fig_dir, pattern = "\\.png$")
  expect_length(png_files, 1L)
  expect_match(out, "\\.png")
})

test_that("writer option is ignored for plain SQL", {
  query <- "SELECT 1 AS x, 2 AS y"
  out <- run_query(query, writer = "vegalite_png")
  # Should produce a table, not an image
  expect_snapshot(cat(out))
})

test_that("invalid writer option produces error", {
  query <- c(
    paste0("SELECT mpg, disp FROM '", data_file, "'"),
    "VISUALISE mpg AS x, disp AS y",
    "DRAW point"
  )
  out <- run_query(query, writer = "ggplot2")
  expect_match(out, "Unsupported writer")
})

test_that("we can knit a mixed-chunk document", {
  skip_if_not_installed("withr")

  # Create a temporary working directory that will be deleted after this test
  dir <- withr::local_tempdir()
  withr::local_dir(dir)

  # We're copying the test file to working directory so side-effects,
  # like creating new figure folders, are contained
  basename <- "test_chunks.qmd"
  doc <- system.file(basename, package = "ggsql")
  in_file <- file.path(dir, basename)
  file.copy(doc, in_file)

  out_file <- file.path(dir, "test_chunks.md")

  withr::local_options(list(knitr.in.progress = TRUE))
  knitr::opts_knit$set(rmarkdown.pandoc.to = "html")
  withr::defer(knitr::opts_knit$set(rmarkdown.pandoc.to = NULL))
  out <- knitr::knit(input = in_file, output = out_file, quiet = TRUE)
  expect_equal(out_file, out)
  expect_true(file.exists(out))

  # Check that visualization was rendered (contains ggsql-vega custom element)
  content <- readLines(out)
  expect_true(any(grepl("ggsql_vega", content)))
})

# read the DESCRIPTION file
desc <- read.dcf("DESCRIPTION")

if (!"SystemRequirements" %in% colnames(desc)) {
  fmt <- c(
    "`SystemRequirements` not found in `DESCRIPTION`.",
    "Please specify `SystemRequirements: Cargo (Rust's package manager), rustc`"
  )
  stop(paste(fmt, collapse = "\n"))
}

# extract system requirements
sysreqs <- desc[, "SystemRequirements"]

# check that cargo and rustc is found
if (!grepl("cargo", sysreqs, ignore.case = TRUE)) {
  stop(
    "You must specify `Cargo (Rust's package manager)` in your `SystemRequirements`"
  )
}

if (!grepl("rustc", sysreqs, ignore.case = TRUE)) {
  stop(
    "You must specify `Cargo (Rust's package manager), rustc` in your `SystemRequirements`"
  )
}

# split into parts
parts <- strsplit(sysreqs, ", ")[[1]]

# identify which is the rustc
rustc_ver <- parts[grepl("rustc", parts)]

# perform checks for the presence of rustc and cargo on the OS
no_cargo_msg <- c(
  "----------------------- [CARGO NOT FOUND]--------------------------",
  "The 'cargo' command was not found on the PATH. Please install Cargo",
  "from: https://www.rust-lang.org/tools/install",
  "",
  "Alternatively, you may install Cargo from your OS package manager:",
  " - Debian/Ubuntu: apt-get install cargo",
  " - Fedora/CentOS: dnf install cargo",
  " - macOS: brew install rust",
  "-------------------------------------------------------------------"
)

no_rustc_msg <- c(
  "----------------------- [RUST NOT FOUND]---------------------------",
  "The 'rustc' compiler was not found on the PATH. Please install",
  paste(rustc_ver, "or higher from:"),
  "https://www.rust-lang.org/tools/install",
  "",
  "Alternatively, you may install Rust from your OS package manager:",
  " - Debian/Ubuntu: apt-get install rustc",
  " - Fedora/CentOS: dnf install rustc",
  " - macOS: brew install rust",
  "-------------------------------------------------------------------"
)

# Add {user}/.cargo/bin to path before checking
new_path <- paste0(
  Sys.getenv("PATH"),
  ":",
  paste0(Sys.getenv("HOME"), "/.cargo/bin")
)

# set the path with the new path
Sys.setenv("PATH" = new_path)

# check for rustc installation
rustc_version <- tryCatch(
  system("rustc --version", intern = TRUE),
  error = function(e) {
    stop(paste(no_rustc_msg, collapse = "\n"))
  }
)

# check for cargo installation
cargo_version <- tryCatch(
  system("cargo --version", intern = TRUE),
  error = function(e) {
    stop(paste(no_cargo_msg, collapse = "\n"))
  }
)

# helper function to extract versions
extract_semver <- function(ver) {
  if (grepl("\\d+\\.\\d+(\\.\\d+)?", ver)) {
    sub(".*?(\\d+\\.\\d+(\\.\\d+)?).*", "\\1", ver)
  } else {
    NA
  }
}

# get the MSRV
msrv <- extract_semver(rustc_ver)

# extract current version
current_rust_version <- extract_semver(rustc_version)

# perform check
if (!is.na(msrv)) {
  # -1 when current version is later
  # 0 when they are the same
  # 1 when MSRV is newer than current
  is_msrv <- utils::compareVersion(msrv, current_rust_version)
  if (is_msrv == 1) {
    fmt <- paste0(
      "\n------------------ [UNSUPPORTED RUST VERSION]------------------\n",
      "- Minimum supported Rust version is %s.\n",
      "- Installed Rust version is %s.\n",
      "---------------------------------------------------------------"
    )
    stop(sprintf(fmt, msrv, current_rust_version))
  }
}

# print the versions
versions_fmt <- "Using %s\nUsing %s"
message(sprintf(versions_fmt, cargo_version, rustc_version))

#' Validate a ggsql query
#'
#' Checks query syntax and semantics without executing SQL. Returns a
#' validation result that can be inspected for errors and warnings.
#'
#' @param query A ggsql query string.
#' @param x A `ggsql_validated` object
#'
#' @return A `ggsql_validated` object for  `ggsql_validate()`. A boolean for
#' `ggsql_has_visual()` and `ggsql_is_valid()`
#'
#' @export
#'
#' @examples
#' result <- ggsql_validate("SELECT 1 AS x, 2 AS y VISUALISE x, y DRAW point")
#' result
#'
ggsql_validate <- function(query) {
  check_string(query, allow_empty = FALSE)
  result <- ggsql_validate_impl(query)

  result$errors <- jsonlite::fromJSON(result$errors_json)
  result$warnings <- jsonlite::fromJSON(result$warnings_json)
  result$errors_json <- NULL
  result$warnings_json <- NULL

  structure(result, class = "ggsql_validated")
}

#' @export
print.ggsql_validated <- function(x, ...) {
  status <- if (x$valid) "valid" else "invalid"
  cli::cli_text("<ggsql_validated> [{status}]")
  if (x$has_visual) {
    cli::cli_bullets(c("*" = "Has VISUALISE clause"))
  }
  if (length(x$errors) > 0 && NROW(x$errors) > 0) {
    cli::cli_text("Errors:")
    for (i in seq_len(NROW(x$errors))) {
      cli::cli_bullets(c("x" = x$errors$message[i]))
    }
  }
  if (length(x$warnings) > 0 && NROW(x$warnings) > 0) {
    cli::cli_text("Warnings:")
    for (i in seq_len(NROW(x$warnings))) {
      cli::cli_bullets(c("!" = x$warnings$message[i]))
    }
  }
  invisible(x)
}

#' @rdname ggsql_validate
#' @export
ggsql_has_visual <- function(x) {
  x$has_visual
}

#' @rdname ggsql_validate
#' @export
ggsql_is_valid <- function(x) {
  x$valid
}

test_that("ggsql_validate returns a ggsql_validated object", {
  v <- ggsql_validate(
    "SELECT 1 AS x, 2 AS y VISUALISE DRAW point MAPPING x AS x, y AS y"
  )
  expect_s3_class(v, "ggsql_validated")
  expect_true(v$has_visual)
  expect_true(v$valid)
})

test_that("ggsql_validate detects errors", {
  v <- ggsql_validate("SELECT 1 AS x VISUALISE x DRAW point")
  expect_false(v$valid)
  expect_true(NROW(v$errors) > 0)
})

test_that("ggsql_validate handles pure SQL", {
  v <- ggsql_validate("SELECT 1 AS x")
  expect_false(v$has_visual)
  expect_true(v$valid)
})

test_that("ggsql_has_visual works", {
  v <- ggsql_validate("SELECT 1 AS x VISUALISE x DRAW point")
  expect_true(ggsql_has_visual(v))

  v2 <- ggsql_validate("SELECT 1 AS x")
  expect_false(ggsql_has_visual(v2))
})

test_that("ggsql_is_valid works", {
  v <- ggsql_validate(
    "SELECT 1 AS x, 2 AS y VISUALISE DRAW point MAPPING x AS x, y AS y"
  )
  expect_true(ggsql_is_valid(v))
})

test_that("print.ggsql_validated works", {
  v <- ggsql_validate(
    "SELECT 1 AS x, 2 AS y VISUALISE DRAW point MAPPING x AS x, y AS y"
  )
  expect_invisible(print(v))
})

# AUTO-GENERATED by tools/update-vega.sh — do not edit by hand.
# To update, change the version variables in tools/update-vega.sh and rerun it.

vega_version <- "6.2.0"
vega_lite_version <- "6.4.2"
vega_embed_version <- "7.1.0"

# Standalone file: do not edit by hand
# Source: https://github.com/r-lib/rlang/blob/HEAD/R/standalone-types-check.R
# Generated by: usethis::use_standalone("r-lib/rlang", "types-check")
# ----------------------------------------------------------------------
#
# ---
# repo: r-lib/rlang
# file: standalone-types-check.R
# last-updated: 2023-03-13
# license: https://unlicense.org
# dependencies: standalone-obj-type.R
# imports: rlang (>= 1.1.0)
# ---
#
# ## Changelog
# 2026-03-30:
# - `check_name()` has been removed from the standalone file because of a
#   conflict with `recipes::check_name()`.
#
# 2026-03-17:
# - `check_bool()`, `check_string()`, `check_number_decimal()`,
#   `check_number_whole()`, and `check_data_frame()` are now exported
#   from rlang. Language type checkers (`check_symbol()`,
#   `check_call()`, etc.), column type checkers (`check_character()`,
#   `check_logical()`), and `check_arg()` remain in the standalone file.
# - `check_formula()` now requires an evaluated formula, not a defused one.
#
# 2025-09-19:
# - `check_logical()` gains an `allow_na` argument (@jonthegeek, #1724)
#
# 2024-08-15:
# - `check_character()` gains an `allow_na` argument (@martaalcalde, #1724)
#
# 2023-03-13:
# - Improved error messages of number checkers (@teunbrand)
# - Added `allow_infinite` argument to `check_number_whole()` (@mgirlich).
# - Added `check_data_frame()` (@mgirlich).
#
# 2023-03-07:
# - Added dependency on rlang (>= 1.1.0).
#
# 2023-02-15:
# - Added `check_logical()`.
#
# - `check_bool()`, `check_number_whole()`, and
#   `check_number_decimal()` are now implemented in C.
#
# - For efficiency, `check_number_whole()` and
#   `check_number_decimal()` now take a `NULL` default for `min` and
#   `max`. This makes it possible to bypass unnecessary type-checking
#   and comparisons in the default case of no bounds checks.
#
# 2022-10-07:
# - `check_number_whole()` and `_decimal()` no longer treat
#   non-numeric types such as factors or dates as numbers.  Numeric
#   types are detected with `is.numeric()`.
#
# 2022-10-04:
# - Added `check_name()` that forbids the empty string.
#   `check_string()` allows the empty string by default.
#
# 2022-09-28:
# - Removed `what` arguments.
# - Added `allow_na` and `allow_null` arguments.
# - Added `allow_decimal` and `allow_infinite` arguments.
# - Improved errors with absent arguments.
#
#
# 2022-09-16:
# - Unprefixed usage of rlang functions with `rlang::` to
#   avoid onLoad issues when called from rlang (#1482).
#
# 2022-08-11:
# - Added changelog.
#
# nocov start

check_symbol <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_symbol(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a symbol",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_arg <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_symbol(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "an argument name",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_call <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_call(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a defused call",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_environment <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_environment(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "an environment",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_function <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_function(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a function",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_closure <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_closure(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "an R function",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_formula <- function(
  x,
  ...,
  allow_null = FALSE,
  allow_unevaluated = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
    scoped <- if (allow_unevaluated) NULL else TRUE
    if (is_formula(x, scoped = scoped)) {
      return(invisible(NULL))
    }
    if (!allow_unevaluated && is_formula(x)) {
      cli::cli_abort(
        "{.arg {arg}} must be an evaluated formula, not a defused one.",
        arg = arg,
        call = call
      )
    }
  }

  stop_input_type(
    x,
    "a formula",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}


# Vectors -----------------------------------------------------------------

# TODO: Figure out what to do with logical `NA` and `allow_na = TRUE`

check_character <- function(
  x,
  ...,
  allow_na = TRUE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_character(x)) {
      if (!allow_na && any(is.na(x))) {
        abort(
          sprintf("`%s` can't contain NA values.", arg),
          arg = arg,
          call = call
        )
      }

      return(invisible(NULL))
    }

    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a character vector",
    ...,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_logical <- function(
  x,
  ...,
  allow_na = TRUE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_logical(x)) {
      if (!allow_na && any(is.na(x))) {
        abort(
          sprintf("`%s` can't contain NA values.", arg),
          arg = arg,
          call = call
        )
      }
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a logical vector",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

# nocov end

#' @noRd
Spec <- R6::R6Class(
  "Spec",
  cloneable = FALSE,
  public = list(
    .ptr = NULL,

    initialize = function(ptr) {
      self$.ptr <- ptr
    },

    print = function(...) {
      json <- ggsql_render(vegalite_writer(), self)
      widget <- ggsql_widget(json)
      print(widget)
    }
  )
)

#' @importFrom utils str
#' @export
str.Spec <- function(object, ...) {
  m <- ggsql_metadata(object)
  cli::cli_text("<ggsql_spec>")
  cli::cli_bullets(c(
    "*" = "Rows: {m$rows}",
    "*" = "Columns: {paste(m$columns, collapse = ', ')}",
    "*" = "Layers: {m$layer_count}"
  ))
  invisible()
}
#' @importFrom knitr knit_print
#' @export
knit_print.Spec <- function(x, ..., inline = FALSE) {
  options <- knitr::opts_current$get()
  writer_type <- options$writer %||% "vegalite"

  switch(
    writer_type,
    vegalite = {
      json <- ggsql_render(vegalite_writer(), x)
      widget <- ggsql_widget(json)
      knitr::knit_print(widget, options = options)
    },
    vegalite_svg = ,
    vegalite_png = {
      write_static_figure(x, sub("vegalite_", "", writer_type), options)
    },
    cli::cli_abort("unknown writer {.val {writer_type}}")
  )
}

#' Utility functions for visualization specifications
#'
#' These functions allow you to extract various information from a `Spec` object
#' returned by [ggsql_execute()].
#'
#' @param spec A `Spec` object as returned by [ggsql_execute()]
#' @param index Layer index
#'
#' @return
#' * `ggsql_metadata`: A list with elements `rows`, `columns`, and `layer_count`
#' * `ggsql_sql`: A character string with the SQL portion of the query
#' * `ggsql_visual`: A character string with the visual portion of the query
#' * `ggsql_layer_count`: An integer giving the number of layers
#' * `ggsql_layer_data`: A data frame, or `NULL` if no data is available for
#' this layer
#' * `ggsql_stat_data`: A data frame, or `NULL` if the layer doesn't use a stat
#' transform
#' * `ggsql_layer_sql`: A character string with the SQL query used by the layer
#' to fetch its data, or `NULL` if the layer doesn't have any data.
#' * `ggsql_stat_sql`: A character string with the SQL query used by the layers
#' stat transform, or `NULL` if the layer doesn't have a stat transform.
#' * `ggsql_warnings`: A data.frame with columns `message`, `line`, and `column`
#' giving the validation warnings for the spec
#'
#' @name spec_utility
#' @rdname spec_utility
#'
#' @examples
#' reader <- duckdb_reader()
#' ggsql_register(reader, mtcars, "cars")
#' spec <- ggsql_execute(reader,
#'   "SELECT * FROM cars VISUALISE mpg AS x DRAW histogram"
#' )
#'
#' ggsql_metadata(spec)
#'
#' ggsql_visual(spec)
#'
NULL

#' @rdname spec_utility
#' @export
ggsql_metadata <- function(spec) {
  check_r6(spec, "Spec")
  list(
    rows = spec$.ptr$metadata_rows(),
    columns = spec$.ptr$metadata_columns(),
    layer_count = spec$.ptr$metadata_layer_count()
  )
}

#' @rdname spec_utility
#' @export
ggsql_sql <- function(spec) {
  check_r6(spec, "Spec")
  spec$.ptr$get_sql()
}

#' @rdname spec_utility
#' @export
ggsql_visual <- function(spec) {
  check_r6(spec, "Spec")
  spec$.ptr$get_visual()
}

#' @rdname spec_utility
#' @export
ggsql_layer_count <- function(spec) {
  check_r6(spec, "Spec")
  spec$.ptr$layer_count()
}

#' @rdname spec_utility
#' @export
ggsql_layer_data <- function(spec, index = 1L) {
  check_r6(spec, "Spec")
  check_number_whole(index, min = 1)
  # Convert R 1-based to Rust 0-based
  ipc_bytes <- spec$.ptr$layer_data_ipc(as.integer(index - 1L))
  if (is.null(ipc_bytes)) {
    return(NULL)
  }
  ipc_to_df(ipc_bytes)
}

#' @rdname spec_utility
#' @export
ggsql_stat_data <- function(spec, index = 1L) {
  check_r6(spec, "Spec")
  check_number_whole(index, min = 1)
  ipc_bytes <- spec$.ptr$stat_data_ipc(as.integer(index - 1L))
  if (is.null(ipc_bytes)) {
    return(NULL)
  }
  ipc_to_df(ipc_bytes)
}

#' @rdname spec_utility
#' @export
ggsql_layer_sql <- function(spec, index = 1L) {
  check_r6(spec, "Spec")
  check_number_whole(index, min = 1)
  spec$.ptr$get_layer_sql(as.integer(index - 1L))
}

#' @rdname spec_utility
#' @export
ggsql_stat_sql <- function(spec, index = 1L) {
  check_r6(spec, "Spec")
  check_number_whole(index, min = 1)
  spec$.ptr$get_stat_sql(as.integer(index - 1L))
}

#' @rdname spec_utility
#' @export
ggsql_warnings <- function(spec) {
  check_r6(spec, "Spec")
  json <- spec$.ptr$warnings_json()
  warnings_list <- jsonlite::fromJSON(json)
  if (length(warnings_list) == 0) {
    return(data.frame(
      message = character(),
      line = integer(),
      column = integer()
    ))
  }
  warnings_list
}

test_that("ggsql_metadata returns correct structure", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  m <- ggsql_metadata(spec)
  expect_type(m, "list")
  expect_equal(m$rows, 32L)
  expect_type(m$columns, "character")
  expect_equal(m$layer_count, 1L)
})

test_that("ggsql_sql returns the SQL portion", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  sql <- ggsql_sql(spec)
  expect_type(sql, "character")
  expect_match(sql, "SELECT")
})

test_that("ggsql_visual returns the VISUALISE portion", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  vis <- ggsql_visual(spec)
  expect_type(vis, "character")
  expect_match(vis, "VISUALISE")
})

test_that("ggsql_layer_count returns integer", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point DRAW line MAPPING mpg AS x, disp AS y"
  )
  expect_equal(ggsql_layer_count(spec), 2L)
})

test_that("ggsql_layer_data returns a data frame", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  df <- ggsql_layer_data(spec, 1L)
  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 32)
})

test_that("ggsql_warnings returns a data frame", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  w <- ggsql_warnings(spec)
  expect_s3_class(w, "data.frame")
})

test_that("spec str method shows metadata", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  expect_invisible(str(spec))
})

test_that("ggsql_widget returns an htmlwidget", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  json <- ggsql:::ggsql_render(ggsql:::vegalite_writer(), spec)
  widget <- ggsql:::ggsql_widget(json)
  expect_s3_class(widget, "htmlwidget")
  expect_s3_class(widget, "ggsql_vega")
  expect_true(!is.null(widget$x$spec))
})

test_that("ggsql_widget renders with a custom element root", {
  reader <- duckdb_reader()
  ggsql_register(reader, mtcars, "cars")
  spec <- ggsql_execute(
    reader,
    "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
  )
  json <- ggsql:::ggsql_render(ggsql:::vegalite_writer(), spec)
  widget <- ggsql:::ggsql_widget(json, width = "225px", height = "360px")
  html <- htmltools::as.tags(widget, standalone = FALSE)

  expect_match(as.character(html), "<ggsql-vega", fixed = TRUE)
  expect_match(as.character(html), "width:225px", fixed = TRUE)
  expect_match(as.character(html), "height:360px", fixed = TRUE)
})

#!/bin/sh
# DuckDB's unity-build C++ translation units produce object files that
# exceed the PE/COFF section limit on x86_64-pc-windows-gnu. The assembler
# big-object format lifts that limit. Kept out of Makevars.win.in so the
# flag text does not land in 00install.out and trip R CMD check's
# non-portable-flag scan.
export CXXFLAGS="${CXXFLAGS} -Wa,-mbig-obj"

---
title: "Getting started"
knitr:
  opts_chunk:
    collapse: true
    comment: "#>"
description: |
  A short introduction to interacting with ggsql from R
vignette: >
  %\VignetteIndexEntry{Getting started}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r setup}
library(ggsql)
```

The ggsql package is an R binding to the [ggsql visualization library](https://ggsql.org). It allows you to set up readers to interact with different data backends, execute queries, and define writers to handle rendering.

In addition to this it also provide a knitr engine so that you can write ggsql blocks in your Rmarkdown and Quarto documents. This part of the package has it's own vignette so head there if that is your main interest.

## Package anatomy
The ggsql R package follows the same division as the main library which is split into a reader, plot, and writer module. You start by creating a reader which interacts with your data backend. You then execute queries against this reader (effectively using the plot module). Lastly, you can render the result using a writer you define.

### Creating a reader
ggsql understands multiple backends, but in this vignette we focus on the DuckDB reader since it is both powerful and doesn't require any additional setup. You create a DuckDB reader using the aptly named `duckdb_reader()` function:

```{r}
ddb <- duckdb_reader()
```

By default it creates a connection to an in-memory database, but you can point it at a database file to connect to that instead:

```{r}
#| eval: false
ddb2 <- duckdb_reader("my/amazing/data.db")
```

While a reader is mainly in service of executing queries against, you can do something else with it as well, e.g. register data to it that will be available to queries on equal footing with any "real" tables in the database:

```{r}
ggsql_register(ddb, gapminder::gapminder, "gapminder")
```

We can see all the tables currently available with: 

```{r}
ggsql_table_names(ddb)
```

And if we wanted to we could fetch the data back using `ggsql_table()` (works for both registered and native tables and views), or unregister it using `ggsql_unregister()` (only for registered tables)

### Executing a query
As mentioned, the main purpose of a reader is to be the backend on which a ggsql query are executed. While it is possible to execute pure SQL queries and get back a data.frame:

```{r}
ggsql_execute_sql(ddb, "
SELECT MAX(pop) AS population, country FROM gapminder
GROUP BY country
ORDER BY population DESC
LIMIT 6
")
```

The main motivation is of course to create visualizations. This vignette will not go into detail with the ggsql syntax. For that, head to the [documentation on the website](https://ggsql.org/syntax/). If you are familiar with ggplot2 you should be able to understand the gist of it.

```{r}
plot <- ggsql_execute(ddb, "
SELECT * FROM gapminder
WHERE year = 2007

VISUALIZE gdpPercap AS x, lifeExp AS y, continent AS stroke, pop AS size
DRAW point
  SETTING fill => null
SCALE size TO (2, 15)
")

str(plot)
```

As you can see, executing a visual query does not in itself produce a plot, but
creates a data structure that contains the information necessary to render it. The analogue is the ggplot object you create when using ggplot2.

You can use this object for introspection, e.g. get the visual part of the query:

```{r}
ggsql_visual(plot)
```

### Rendering a plot
Finally we arrive at what we was here for all along: How to get a plot out of it. ggsql works with the concept of writers which are somewhat analogous to graphics devices in R. They are responsible for figuring out a visual representation of the abstract encoding in the plot specification. For the moment ggsql only comes with a vegalite writer so this is also the only one available in the R package:

```{r}
vlr <- vegalite_writer()
```

Often you don't have to think about this at all since printing the result of a query automatically creates a writer and renders it so that it behaves in the same way as ggplot object (this is also true in Quarto/Rmarkdown documents):

```{r}
plot
```

However, if you are interested in capturing what the writer produces you can call `ggsql_render()` explicitly:

```{r}
ggsql_render(vlr, plot)
```

We might also be interested in capturing the plot as a static image. We can use the `ggsql_save()` function for that which can save the output to either a vegalite json object (like the one above), an SVG or a PNG. For the latter two the plot is rendered using the V8 package and the final image captured.

```{r}
tmp <- tempfile(fileext = ".svg")
ggsql_save(plot, tmp)
readLines(tmp)
```

---
title: "Test chunks"
author: "Test author"
format: html
---

```{r}
library(ggsql)
test_file <- "test.csv"
write.csv(mtcars, test_file)
```

```{ggsql}
SELECT mpg, disp FROM 'test.csv'
VISUALISE mpg AS x, disp AS y
DRAW point
```

```{r}
unlink(test_file)
```

---
title: "The ggsql knitr engine"
knitr:
  opts_chunk:
    collapse: true
    comment: "#>"
description: |
  An introduction to using the ggsql knitr engine in Rmarkdown and 
  Quarto
vignette: >
  %\VignetteIndexEntry{The ggsql knitr engine}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r setup}
library(ggsql)
```

The main selling point of the ggsql R package is arguably it's knitr engine that allows you to add `{ggsql}` blocks to your Rmarkdown and Quarto documents. While ggsql also provides a Jupyter kernel that Quarto can use, Jupyter only allows a single kernel at a time, so if you wish to mix R and/or Python blocks with ggsql visualizations, then the ggsql knitr engine is the way.

This vignette will show you how to set it up and use it.

## Registering the engine
Before knitr knows what to do with a `{ggsql}` code chunk you need to register the ggsql engine. Registration happens automatically when ggsql is loaded, so the only thing required is to load the package in the setup chunk:

```{{r setup}}
library(ggsql) # register the engine

# Other stuff you may want to do
set.seed(7)
```

## The `{ggsql}` chunk
Once you've registered the engine you can now begin to add `{ggsql}` blocks to your document containing ggsql queries:

```{{ggsql}}
VISUALIZE species AS x FROM ggsql:penguins
DRAW bar
```

Which, when rendered will result in:

```{ggsql}
VISUALIZE species AS x FROM ggsql:penguins
DRAW bar
```

If the code only contains a pure SQL query, then the table is returned:

```{ggsql}
SELECT COUNT(*) AS count, species FROM ggsql:penguins
GROUP BY species
```

### Chunk options
The ggsql knitr engine responds to all the standard plot related chunk options you are familiar with. However, there are a few exceptions: 

* `dev` and `dev.args`: ggsql does not use the R graphics device system for rendering
* `fig.showtext`: The showtext package works with the graphics device system
* `external` and `sanitize`: ggsql does not use the tikz system for rendering

In addition to the standards there are two new options you can use:

* `connection`
* `writer`

Both will be described below.

The ggsql engine supports inline chunk options, using both the standard SQL comment chararcters (`--`) and the R comment character that many users will find recognizable for chunk options. Either of these, proceeded by the pipe character (`--|` and `#|`) will mark a chunk option line as long as they are the first appearing in the chunk.

#### Defining a backend
By default, a ggsql chunk will provide an in memory DuckDB writer when used with the knitr engine. This is fine as long as you don't need to interact directly with data from your warehouse. If you need something else you can provide a different connection string in the `connection` chunk option. Chunks that use the same connection will use the same writer, so temporary tables and views will be accessible across chunks. To interact directly with a file based DuckDB database living at `/data/users.db` you'd set it like so:

```{{ggsql}}
--| connection: duckdb:///data/users.db
VISUALIZE ...
```

#### Choosing a writer
ggsql doesn't have the concept of graphics devices as you may be familiar with from R. Instead it has *writers* which are responsible for rendering. Currently we only provide a Vega-Lite writer, but it comes in three flavors in the knitr engine: `vegalite`, `vegalite_svg`, and `vegalite_png`. The former renders to a Vega-Lite spec which is then rendered to the final graphic when the page HTML is rendered. The latter two uses Vega-Lite off-screen to render to SVG and PNG respectively, making the final document fully stand-alone. The default for HTML documents is to use the `vegalite` writer and for PDF to use the `vegalite_png` writer, but you can overwrite this using the `writer` option:

```{{ggsql}}
--| writer: vegalite_svg
VISUALIZE ...
```

The `writer` option also affects rendering of `Spec` objects (as returned by `ggsql_execute()`) in standard `{r}` code chunks.

## Communicating with R and Python
The big selling point of the knitr engine over the ggsql Jupyter kernel is its ability to co-exist with R and Python chunks and use data defined in them. This means you can bring data into R, do some manipulation or statistical analysis that would be difficult to express in SQL, and then use ggsql to directly visualize it.

The way this works borrows the syntax from referencing built-in data in ggsql. Above, we used `ggsql:penguins` to refer to the penguins dataset shipped with ggsql. Likewise, you can prefix a table with `r:` and `py:` to refer to data from the documents R and Python chunks respectively:

```{r}
gapminder_2002 <- gapminder::gapminder[gapminder::gapminder$year == 2002, ]
```

```{ggsql}
VISUALIZE gdpPercap AS x, lifeExp AS y FROM r:gapminder_2002
DRAW point
  MAPPING continent AS stroke, pop AS size
  SETTING fill => null
LABEL
  title => 'Rendering data from R'
```

### Getting data back from ggsql
While there is certainly more use from the above direction of communication it is also possible to use tables or views created in ggsql chunks from R and Python. The way this works mirrors how R and Python communicate with each other. The ggsql knitr engine provides a `sql` object in the R and Python environments respectively which can be indexed to return any table or view available in the ggsql backend. If your document uses multiple ggsql backends the `connection` option of the R/Python chunk determines which backend is used to reference into:

```{ggsql}
CREATE VIEW may_airquality AS 
SELECT * FROM ggsql:airquality
WHERE Month = 5
```

```{ggsql}
CREATE TABLE penguin_count AS 
SELECT COUNT(*) AS number, species FROM ggsql:penguins
GROUP BY species
```

```{r}
names(sql)

sql$penguin_count
```

```{r}
#| include: false
# ggsql caches builtin datasets (penguins, airquality) as parquet files in
# the OS temp dir (Rust's `env::temp_dir()`, i.e. `$TMPDIR` or `/tmp`) and
# never cleans them up. Remove them so R CMD check doesn't flag them as
# detritus in the temp directory.
tmp_root <- Sys.getenv("TMPDIR", unset = "/tmp")
unlink(file.path(tmp_root, c("airquality.parquet", "penguins.parquet")))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{ggsql_validate}
\alias{ggsql_validate}
\alias{ggsql_has_visual}
\alias{ggsql_is_valid}
\title{Validate a ggsql query}
\usage{
ggsql_validate(query)

ggsql_has_visual(x)

ggsql_is_valid(x)
}
\arguments{
\item{query}{A ggsql query string.}

\item{x}{A \code{ggsql_validated} object}
}
\value{
A \code{ggsql_validated} object for  \code{ggsql_validate()}. A boolean for
\code{ggsql_has_visual()} and \code{ggsql_is_valid()}
}
\description{
Checks query syntax and semantics without executing SQL. Returns a
validation result that can be inspected for errors and warnings.
}
\examples{
result <- ggsql_validate("SELECT 1 AS x, 2 AS y VISUALISE x, y DRAW point")
result

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reader.R
\name{custom_reader}
\alias{custom_reader}
\title{Create a reader backed by R callbacks}
\usage{
custom_reader(execute_sql, register = NULL, unregister = NULL)
}
\arguments{
\item{execute_sql}{A function \verb{function(sql)} that executes \code{sql} and
returns either a data frame or a raw vector containing Arrow IPC
stream bytes (as produced by \code{nanoarrow::as_nanoarrow_array_stream()}
/ arrow IPC writers).}

\item{register}{Optional \verb{function(name, df, replace)} that registers
\code{df} as a table named \code{name}. \code{replace} is \code{TRUE} if the caller
expects an existing table with the same name to be replaced.}

\item{unregister}{Optional \verb{function(name)} that removes a previously
registered table.}
}
\value{
A \code{Reader} object, usable anywhere the other \verb{*_reader()}
constructors are accepted.
}
\description{
Construct a reader whose behavior is defined entirely by R functions
you supply. This makes it possible to plug in data sources that aren't
provided natively by ggsql (e.g. an in-memory store, a custom HTTP API,
a DBI connection, etc.) without touching the Rust side.
}
\details{
Only \code{execute_sql} is required. If \code{register} or \code{unregister} are
omitted, calling \code{\link[=ggsql_register]{ggsql_register()}} / \code{\link[=ggsql_unregister]{ggsql_unregister()}} on the
returned reader raises an error.
}
\examples{
# A trivial reader backed by a list of data frames in an environment,
# delegating the actual SQL engine to an in-memory DuckDB.
store <- new.env(parent = emptyenv())
backend <- duckdb_reader()
reader <- custom_reader(
  execute_sql = function(sql) ggsql_execute_sql(backend, sql),
  register = function(name, df, replace) {
    store[[name]] <- df
    ggsql_register(backend, df, name, replace = replace)
  },
  unregister = function(name) {
    rm(list = name, envir = store)
    ggsql_unregister(backend, name)
  }
)
ggsql_register(reader, mtcars, "cars")
ggsql_execute_sql(reader, "SELECT mpg, disp FROM cars LIMIT 3")

}
\seealso{
Other readers: 
\code{\link{duckdb_reader}()},
\code{\link{odbc_reader}()},
\code{\link{snowflake_reader}()}
}
\concept{readers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reader.R
\name{odbc_reader}
\alias{odbc_reader}
\title{Create an ODBC reader}
\usage{
odbc_reader(
  dsn = NULL,
  driver = NULL,
  server = NULL,
  database = NULL,
  uid = NULL,
  pwd = NULL,
  ...,
  connection_string = NULL
)
}
\arguments{
\item{dsn}{Name of a data source configured in \code{odbc.ini} / \code{odbcinst.ini}.}

\item{driver}{ODBC driver name (e.g. \code{"{PostgreSQL}"}, \code{"{Snowflake}"}).
Curly brackets are optional.}

\item{server, database, uid, pwd}{Common ODBC parameters.}

\item{...}{Additional named \code{key = value} parameters appended to the
connection string (e.g. \code{Port = 5432}, \code{Warehouse = "COMPUTE_WH"}).}

\item{connection_string}{A full ODBC connection string, e.g.
\code{"Driver={PostgreSQL};Server=localhost;Database=mydb;UID=user;PWD=pass"}.
A leading \verb{odbc://} is accepted and stripped.}
}
\value{
A \code{Reader} object.
}
\description{
Creates a connection to a database through an ODBC driver. Can execute SQL
queries and, where the backend supports it, register R data frames as
temporary tables. Requires an ODBC driver manager (unixODBC/iODBC on Unix,
built into Windows) and the appropriate ODBC driver for the target database
to be installed.
}
\details{
Either pass a full ODBC connection string via \code{connection_string}, or supply
named components (\code{dsn}, \code{driver}, \code{server}, \code{database}, \code{uid}, \code{pwd}, plus
any extra \code{key = value} pairs in \code{...}) and they will be assembled into a
connection string. If \code{connection_string} is supplied, the other named
arguments are ignored.
}
\section{Credentials}{

Connection strings are stored in-memory for the life of the reader. Prefer
configuring credentials through a DSN, \verb{~/.odbc.ini}, or environment
variables rather than hard-coding passwords in scripts.
}

\examples{
\dontrun{
# Using a preconfigured DSN
reader <- odbc_reader(dsn = "mydsn")

# Building a connection string from components
reader <- odbc_reader(
  driver = "{PostgreSQL}",
  server = "localhost",
  database = "mydb",
  uid = "user",
  pwd = "secret",
  Port = 5432
)

# Passing a full connection string
reader <- odbc_reader("Driver={SQLite3};Database=:memory:")
}

}
\seealso{
Other readers: 
\code{\link{custom_reader}()},
\code{\link{duckdb_reader}()},
\code{\link{snowflake_reader}()}
}
\concept{readers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{ggsql_session_reader}
\alias{ggsql_session_reader}
\title{Set the ggsql reader for the current Shiny session}
\usage{
ggsql_session_reader(reader, session = shiny::getDefaultReactiveDomain())
}
\arguments{
\item{reader}{A \code{Reader} object created by \code{\link[=duckdb_reader]{duckdb_reader()}}.}

\item{session}{The Shiny session object. Defaults to the current session.}
}
\value{
The \code{reader}, invisibly.
}
\description{
Registers a \code{\link[=duckdb_reader]{duckdb_reader()}} for use by all \code{\link[=renderGgsql]{renderGgsql()}} outputs in the
current Shiny session. Must be called from within a Shiny server function
(i.e., while a session is active). The reader is automatically cleaned up
when the session ends.
}
\examples{
\dontshow{if (interactive() && requireNamespace("shiny", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(shiny)

ui <- fluidPage(
  ggsqlOutput("chart")
)

server <- function(input, output, session) {
  ggsql_session_reader(duckdb_reader())

  output$chart <- renderGgsql({
    "SELECT * FROM r:mtcars VISUALISE mpg AS x, disp AS y DRAW point"
  })
}

shinyApp(ui, server)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reader.R
\name{snowflake_reader}
\alias{snowflake_reader}
\title{Create a Snowflake reader}
\usage{
snowflake_reader(
  account = NULL,
  warehouse = NULL,
  database = NULL,
  schema = NULL,
  role = NULL,
  user = NULL,
  password = NULL,
  authenticator = NULL,
  connection_name = NULL,
  driver = NULL,
  ...,
  connection_string = NULL
)
}
\arguments{
\item{account}{Snowflake account identifier (e.g. \code{"xy12345"} or
\code{"xy12345.us-east-1"}). Translated to
\verb{Server=\{account\}.snowflakecomputing.com} in the connection string.}

\item{warehouse, database, schema, role}{Snowflake session defaults.}

\item{user, password}{User credentials. Prefer a DSN, \code{connection_name}, or
\code{authenticator = "externalbrowser"} over hard-coded passwords.}

\item{authenticator}{Snowflake authenticator (e.g. \code{"externalbrowser"},
\code{"snowflake_jwt"}, \code{"oauth"}).}

\item{connection_name}{Named entry in \verb{~/.snowflake/connections.toml} whose
fields will fill in the remaining connection parameters.}

\item{driver}{Override the ODBC driver name (defaults to \code{"Snowflake"}).}

\item{...}{Additional named \code{key = value} parameters appended to the
connection string.}

\item{connection_string}{A full raw connection string, bypassing the named
arguments. \code{Driver={Snowflake};} is prepended if it isn't already present.}
}
\value{
A \code{Reader} object.
}
\description{
Convenience constructor for Snowflake connections. Uses the ODBC reader
under the hood with the \code{Snowflake} driver, and takes advantage of the
dedicated Snowflake handling in the ggsql Rust core:
}
\details{
Special handling of Snowflake includes:
\itemize{
\item If \code{connection_name} is supplied (or \verb{ConnectionName=} appears in the
connection string), it is resolved against \verb{~/.snowflake/connections.toml}.
\item When running inside Posit Workbench, an OAuth token is automatically
injected if one is available.
\item Schema introspection uses Snowflake's \verb{SHOW DATABASES / SCHEMAS / TABLES}
commands rather than \code{information_schema}.
}

Requires the Snowflake ODBC driver to be installed on the system.
}
\examples{
\dontrun{
# Using a named connection from ~/.snowflake/connections.toml
reader <- snowflake_reader(connection_name = "my_workbench")

# Browser-based SSO
reader <- snowflake_reader(
  account = "xy12345.us-east-1",
  user = "alice@example.com",
  authenticator = "externalbrowser",
  warehouse = "COMPUTE_WH",
  database = "ANALYTICS",
  schema = "PUBLIC",
  role = "ANALYST"
)
}

}
\seealso{
Other readers: 
\code{\link{custom_reader}()},
\code{\link{duckdb_reader}()},
\code{\link{odbc_reader}()}
}
\concept{readers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writer.R
\name{ggsql_render}
\alias{ggsql_render}
\title{Render a spec with a writer}
\usage{
ggsql_render(writer, spec)
}
\arguments{
\item{writer}{A \code{Writer} object created by e.g. \code{\link[=vegalite_writer]{vegalite_writer()}}.}

\item{spec}{A \code{Spec} object returned by \code{\link[=ggsql_execute]{ggsql_execute()}}.}
}
\value{
Writer dependent:
\itemize{
\item \code{vegalite_writer}: A string holding the vegalite JSON representation of the
visualization
}
}
\description{
This function takes a \code{Spec} object as returned by \code{\link[=ggsql_execute]{ggsql_execute()}} and
renders it with the provided writer.
}
\examples{
reader <- duckdb_reader()
ggsql_register(reader, mtcars, "cars")
spec <- ggsql_execute(reader,
  "SELECT * FROM cars VISUALISE mpg AS x DRAW histogram"
)

ggsql_render(vegalite_writer(), spec)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggsql-package.R
\docType{package}
\name{ggsql-package}
\alias{ggsql}
\alias{ggsql-package}
\title{ggsql: Grammar of Graphics for SQL}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Allows you to write queries that combine SQL (Structured Query Language) data retrieval with visualization specifications in a single, composable syntax. The 'ggsql' package binds directly with the 'ggsql' 'Rust' library and allows you to set up readers and writers and execute queries against it. The package also offers 'knitr' and 'shiny' integration allowing the user to use 'ggsql' in both frameworks.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://r.ggsql.org}
  \item \url{https://github.com/posit-dev/ggsql-r}
  \item Report bugs at \url{https://github.com/posit-dev/ggsql-r/issues}
}

}
\author{
\strong{Maintainer}: Thomas Lin Pedersen \email{thomas.pedersen@posit.co} (\href{https://orcid.org/0000-0002-5147-4711}{ORCID})

Authors:
\itemize{
  \item George Stagg \email{george.stagg@posit.co} (\href{https://orcid.org/0009-0006-3173-9846}{ORCID})
  \item Teun van den Brand \email{teun.vandenbrand@posit.co} (\href{https://orcid.org/0000-0002-9335-7468}{ORCID})
}

Other contributors:
\itemize{
  \item Posit, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{ggsql_save}
\alias{ggsql_save}
\title{Save a ggsql spec to a file}
\usage{
ggsql_save(spec, file, width = 600, height = 400)
}
\arguments{
\item{spec}{A \code{Spec} object returned by \code{\link[=ggsql_execute]{ggsql_execute()}}.}

\item{file}{Output file path. Extension determines format: \code{.svg}, \code{.png}, or
\code{.json}.}

\item{width}{Width in pixels.}

\item{height}{Height in pixels.}
}
\value{
\code{file}, invisibly.
}
\description{
This function renders a specification and returns it either as a Vegalite
json string, an SVG or a PNG. For the latter two, the Vegalite JSON is
rendered to SVG using the V8 package and, potentially, converted to PNG using
the rsvg package.
}
\examples{
reader <- duckdb_reader()
ggsql_register(reader, mtcars, "cars")
spec <- ggsql_execute(reader,
  "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
)
spec_file <- tempfile(fileext = ".json")
ggsql_save(spec, spec_file)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writer.R
\name{vegalite_writer}
\alias{vegalite_writer}
\title{Create a Vega-Lite writer}
\usage{
vegalite_writer()
}
\value{
A \code{Writer} object.
}
\description{
This function creates a vegalite writer which is currently the only writer
type for ggsql
}
\examples{
vegalite_writer()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reader.R
\name{duckdb_reader}
\alias{duckdb_reader}
\title{Create a DuckDB reader}
\usage{
duckdb_reader(database = NULL)
}
\arguments{
\item{database}{Path to a DuckDB database file, or \code{NULL} (the default)
for an in-memory database.}
}
\value{
A \code{Reader} object.
}
\description{
Creates a DuckDB database connection that can execute SQL queries and
register data frames as queryable tables. The default creates an empty
in-memory database but you can also pass the path to a DuckDB database to
directly interact with that.
}
\examples{
reader <- duckdb_reader()
ggsql_register(reader, mtcars, "cars")
df <- ggsql_execute_sql(reader, "SELECT mpg, disp FROM cars LIMIT 5")

}
\seealso{
Other readers: 
\code{\link{custom_reader}()},
\code{\link{odbc_reader}()},
\code{\link{snowflake_reader}()}
}
\concept{readers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reader.R
\name{ggsql_register}
\alias{ggsql_register}
\alias{ggsql_unregister}
\alias{ggsql_table}
\alias{ggsql_table_names}
\title{Register and unregisters a data frame as a queryable table}
\usage{
ggsql_register(reader, df, name, replace = FALSE)

ggsql_unregister(reader, name)

ggsql_table(reader, name)

ggsql_table_names(reader)
}
\arguments{
\item{reader}{A \code{Reader} object created by e.g. \code{\link[=duckdb_reader]{duckdb_reader()}}.}

\item{df}{A data frame to register.}

\item{name}{The name of the table.}

\item{replace}{If \code{TRUE}, replace an existing table with the same name.
Defaults to \code{FALSE}.}
}
\value{
\code{reader} for \code{ggsql_register()} and \code{ggsql_unregister()}.
\code{ggsql_table} returns a data.frame if the table exists and \code{NULL} if not.
\code{ggsql_table_names} return a character vector.
}
\description{
After registration, the data frame can be queried by name in SQL statements.
You can use \code{ggsql_table} to extract tables from the reader (both registered
ones and those native to the backend) and \code{ggsql_table_names} to get a vector
of all the tables in reader.
}
\examples{
reader <- duckdb_reader()
ggsql_register(reader, mtcars, "cars")

ggsql_table_names(reader)

ggsql_table(reader, "cars")

ggsql_unregister(reader, "cars")

ggsql_table_names(reader)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{ggsqlOutput}
\alias{ggsqlOutput}
\alias{renderGgsql}
\title{Shiny bindings for ggsql}
\usage{
ggsqlOutput(outputId, width = "100\%", height = "400px")

renderGgsql(expr, reader = NULL, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{outputId}{Output variable to read from.}

\item{width, height}{CSS dimensions for the output container.}

\item{expr}{An expression that returns a ggsql query string or a \code{Spec}
object. Strings may contain \code{r:varname} references that resolve variables
from the expression's local scope (see Examples).}

\item{reader}{A \code{Reader} object created by \code{\link[=duckdb_reader]{duckdb_reader()}}. When \code{NULL}
(the default), the session reader set by \code{\link[=ggsql_session_reader]{ggsql_session_reader()}} is used.}

\item{env}{The environment in which to evaluate \code{expr}.}

\item{quoted}{Logical. Is \code{expr} a quoted expression?}
}
\value{
\code{ggsqlOutput()} returns a Shiny UI element. \code{renderGgsql()} returns
a Shiny render function.
}
\description{
Render ggsql visualizations in a Shiny application. \code{renderGgsql()} accepts
either a ggsql query string or a \code{Spec} object (returned by
\code{\link[=ggsql_execute]{ggsql_execute()}}). When given a string, it validates and executes the query
against \code{reader}.
}
\examples{
\dontshow{if (interactive() && requireNamespace("shiny", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(shiny)

ui <- fluidPage(
  ggsqlOutput("chart")
)

server <- function(input, output, session) {
  ggsql_session_reader(duckdb_reader())

  output$chart <- renderGgsql({
    "SELECT * FROM r:mtcars VISUALISE mpg AS x, disp AS y DRAW point"
  })
}

shinyApp(ui, server)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reader.R
\name{ggsql_execute}
\alias{ggsql_execute}
\alias{ggsql_execute_sql}
\title{Execute a ggsql query}
\usage{
ggsql_execute(reader, query)

ggsql_execute_sql(reader, query)
}
\arguments{
\item{reader}{A \code{Reader} object created by e.g. \code{\link[=duckdb_reader]{duckdb_reader()}} or
\code{\link[=odbc_reader]{odbc_reader()}}.}

\item{query}{A ggsql query string (SQL + VISUALISE clause).}
}
\value{
\code{ggsql_execute()} returns \code{Spec} object. \code{ggsql_execute_sql()}
returns a data frame or \code{NULL}
}
\description{
Parses the query, and execute it against the reader's database. Returns
either a visualization specification ready for rendering (\code{ggsql_execute}) or
a data frame with the query result (\code{ggsql_execute_sql}).
}
\examples{
reader <- duckdb_reader()
ggsql_register(reader, mtcars, "cars")
spec <- ggsql_execute(reader,
  "SELECT * FROM cars VISUALISE mpg AS x, disp AS y DRAW point"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec.R
\name{spec_utility}
\alias{spec_utility}
\alias{ggsql_metadata}
\alias{ggsql_sql}
\alias{ggsql_visual}
\alias{ggsql_layer_count}
\alias{ggsql_layer_data}
\alias{ggsql_stat_data}
\alias{ggsql_layer_sql}
\alias{ggsql_stat_sql}
\alias{ggsql_warnings}
\title{Utility functions for visualization specifications}
\usage{
ggsql_metadata(spec)

ggsql_sql(spec)

ggsql_visual(spec)

ggsql_layer_count(spec)

ggsql_layer_data(spec, index = 1L)

ggsql_stat_data(spec, index = 1L)

ggsql_layer_sql(spec, index = 1L)

ggsql_stat_sql(spec, index = 1L)

ggsql_warnings(spec)
}
\arguments{
\item{spec}{A \code{Spec} object as returned by \code{\link[=ggsql_execute]{ggsql_execute()}}}

\item{index}{Layer index}
}
\value{
\itemize{
\item \code{ggsql_metadata}: A list with elements \code{rows}, \code{columns}, and \code{layer_count}
\item \code{ggsql_sql}: A character string with the SQL portion of the query
\item \code{ggsql_visual}: A character string with the visual portion of the query
\item \code{ggsql_layer_count}: An integer giving the number of layers
\item \code{ggsql_layer_data}: A data frame, or \code{NULL} if no data is available for
this layer
\item \code{ggsql_stat_data}: A data frame, or \code{NULL} if the layer doesn't use a stat
transform
\item \code{ggsql_layer_sql}: A character string with the SQL query used by the layer
to fetch its data, or \code{NULL} if the layer doesn't have any data.
\item \code{ggsql_stat_sql}: A character string with the SQL query used by the layers
stat transform, or \code{NULL} if the layer doesn't have a stat transform.
\item \code{ggsql_warnings}: A data.frame with columns \code{message}, \code{line}, and \code{column}
giving the validation warnings for the spec
}
}
\description{
These functions allow you to extract various information from a \code{Spec} object
returned by \code{\link[=ggsql_execute]{ggsql_execute()}}.
}
\examples{
reader <- duckdb_reader()
ggsql_register(reader, mtcars, "cars")
spec <- ggsql_execute(reader,
  "SELECT * FROM cars VISUALISE mpg AS x DRAW histogram"
)

ggsql_metadata(spec)

ggsql_visual(spec)

}
